# Quick and simple scraper to pull some data from OPGG using multisearch

# Author  : ShoobyDoo
# Date    : 2023-07-05
# License : BSD-3-Clause

from datetime import datetime
from opgg.v1.league_stats import Tier


class RankEntry:
    """
    Represents a rank entry.
    
    ### Properties:
        `game_type: str` - The type of game associated with the rank entry\n
        `rank_info: Tier` - Information about the tier of the rank entry\n
        `created_at: datetime` - Timestamp of when the rank entry was created\n
    """
    def __init__(self,
                 game_type: str,
                 rank_info: Tier,
                 created_at: datetime) -> None:
        self._game_type = game_type
        self._rank_info = rank_info
        self._created_at = created_at
    
    @property
    def game_type(self) -> str:
        """
        A `str` representing the game type (Solo, Flex, etc.)
        """
        return self._game_type
    
    @property
    def rank_info(self) -> Tier:
        """
        A `Tier` object representing the ranked information for a given queue
        """
        return self._rank_info
    
    @property
    def created_at(self) -> datetime:
        """
        A `datetime` object representing when the rank was created (?)\n
        Unclear what this date represents.
        """
        return self._created_at
    

class Season:
    """
    Represents a season.
    
    ### Properties:
        `season_id: int` - Unique identifier for the season\n
        `tier_info: Tier` - Information about the tier of the season\n
        `rank_entries: list[RankEntry]` - List of rank entries for the season\n
        `created_at: datetime` - Timestamp of when the season was created\n
    """
    def __init__(self,
                 season_id: int,
                 tier_info: Tier,
                 rank_entries: list[RankEntry],
                 created_at: datetime) -> None:
        self._season_id = season_id
        self._tier_info = tier_info
        self._rank_entries = rank_entries
        self._created_at = created_at
        
    @property
    def season_id(self) -> int:
        """
        An `int` representing the season id
        """
        return self._season_id
    
    @season_id.setter
    def season_id(self, value: int) -> None:
        self._season_id = value
        
    @property
    def tier_info(self) -> Tier:
        """
        A `Tier` object representing the tier info
        """
        return self._tier_info
    
    @tier_info.setter
    def tier_info(self, value: Tier) -> None:
        self._tier_info = value
    
    @property
    def rank_entries(self) -> list[RankEntry]:
        """
        A `list[RankEntry]` objects containing rank entries for a given season
        """
        return self._rank_entries
    
    @property
    def created_at(self) -> datetime:
        """
        A `datetime` object representing when the season was created
        """
        return self._created_at
     
    @created_at.setter
    def created_at(self, value: datetime) -> None:
        self._created_at = value
    
    def __repr__(self) -> str:
        return f"Season(season={self._season_id}, tier_info={self._tier_info})"
    

class SeasonInfo:
    """
    Represents information about a specific season.\n
    
    ### Properties:
        `id: int` - Unique identifier for the season\n
        `value: int` - Numerical value associated with the season\n
        `display_value: int` - The actual display value\n
        `split: int` - Split information relevant to the season\n
        `is_preseason: bool` - Indicator whether the season is in the preseason\n
    """
    
    def __init__(self,
                 id: int,
                 value: int,
                 display_value: int,
                 split: int,
                 is_preseason: bool) -> None:
        self._id = id
        self._value = value
        self._display_value = display_value
        self._split = split
        self._is_preseason = is_preseason
    
    @property
    def id(self) -> int:
        """
        An `int` representing the season id
        """
        return self._id
    
    @property
    def value(self) -> int:
        """
        An `int` representing the season value
        """
        return self._value
    
    @property
    def display_value(self) -> int:
        """
        An `int` representing the season display value
        """
        return self._display_value
    
    @property
    def split(self) -> int:
        """
        An `int` representing the split in a given season
        """
        return self._split
    
    @property
    def is_preseason(self) -> bool:
        """
        A `bool` representing if the season is preseason
        """
        return self._is_preseason

    def __repr__(self) -> str:
        return f"SeasonInfo(display_value={self._display_value}, is_preseason={self._is_preseason})"


from typing import Optional
from datetime import datetime
from pydantic import BaseModel, Field, HttpUrl

from opgg.v2.season import League, Season, TierInfo
from opgg.v2.champion import MostChampions


class Summoner(BaseModel):
    """Represents a League of Legends summoner."""

    # Base summoner attributes
    id: int
    """The internal OPGG id."""

    summoner_id: str
    """The summoner ID."""

    acct_id: str
    """The account ID used by Riot's API."""

    puuid: str
    """The PUUID (`P`layer `U`niversally `U`nique `ID`entifier)."""

    game_name: str
    """The summoner's display name."""

    tagline: str
    """The summoner's tagline (e.g. NA1, EUW)."""

    name: str
    """The summoner's previous name before Riot ID system."""

    internal_name: str
    """OPGG's internal name format (lowercase, no spaces)."""

    profile_image_url: HttpUrl
    """URL to the summoner's profile icon image."""

    level: int
    """The summoner's current level."""

    updated_at: datetime
    """When the summoner's data was last updated."""

    renewable_at: datetime
    """When the summoner's data can be updated again."""

    revision_at: datetime
    """When the summoner's data was last revised."""

    # Search result specific attributes
    solo_tier_info: Optional[TierInfo] = Field(default=None)
    """Solo tier ranking information."""

    team_info: Optional[dict] = Field(default=None)
    """Team information."""

    # Full profile specific attributes
    previous_seasons: Optional[list[Season]] = Field(default=None)
    """Previous season information."""

    league_stats: Optional[list[League]] = Field(default=None)
    """League statistics."""

    most_champions: Optional[MostChampions] = Field(default=None)
    """Most played champions."""

    class Config:
        """Pydantic model configuration."""

        arbitrary_types_allowed = True
        json_encoders = {datetime: lambda v: v.isoformat()}

    @property
    def is_full_profile(self) -> bool:
        """Check if this instance contains full profile data."""
        return all(
            [
                self.previous_seasons is not None,
                self.league_stats is not None,
                self.most_champions is not None,
            ]
        )



from pydantic import BaseModel

from typing import Any, Optional
from datetime import datetime


class ChampionStats(BaseModel):
    id: int
    play: int
    win: int
    lose: int
    game_length_second: int
    kill: int
    death: int
    assist: int
    gold_earned: int
    minion_kill: int
    neutral_minion_kill: int
    damage_taken: int
    damage_dealt_to_champions: int
    double_kill: int = 0
    triple_kill: int = 0
    quadra_kill: int = 0
    penta_kill: int = 0
    vision_wards_bought_in_game: int = 0
    op_score: int
    snowball_throws: Optional[int] = None
    snowball_hits: Optional[int] = None

    @property
    def winrate(self) -> int:
        """`[Computed Property]` Returns the winrate percentage of the champion."""
        return round(self.win / self.play * 100) if self.play != 0 else 0


class MostChampions(BaseModel):
    game_type: str
    season_id: int
    year: Optional[int] = None
    play: int
    win: int
    lose: int
    champion_stats: list[ChampionStats]


class Price(BaseModel):
    currency: str
    cost: int


class Skin(BaseModel):
    id: int
    champion_id: int
    name: str
    has_chromas: bool
    splash_image: str
    loading_image: str
    tiles_image: str
    centered_image: str
    skin_video_url: str | None
    prices: list[Price] | None
    sales: Any | None  # We don't have information about the structure of sales
    release_date: datetime | None


class Info(BaseModel):
    attack: int
    defense: int
    magic: int
    difficulty: int


class Stats(BaseModel):
    hp: float
    hpperlevel: float
    mp: float
    mpperlevel: float
    movespeed: float
    armor: float
    armorperlevel: float
    spellblock: float
    spellblockperlevel: float
    attackrange: float
    hpregen: float
    hpregenperlevel: float
    mpregen: float
    mpregenperlevel: float
    crit: float
    critperlevel: float
    attackdamage: float
    attackdamageperlevel: float
    attackspeed: float
    attackspeedperlevel: float


class Passive(BaseModel):
    name: str
    description: str
    image_url: str
    video_url: str


class Spell(BaseModel):
    key: str
    name: str
    description: str
    max_rank: int
    range_burn: list[int]
    cooldown_burn: list[float]
    cooldown_burn_float: list[float]
    cost_burn: list[int]
    tooltip: str
    image_url: str
    video_url: str


class Evolve(BaseModel):
    key: str
    name: str
    image_url: str


class Champion(BaseModel):
    id: int
    key: str
    name: str
    image_url: str
    evolve: list[Evolve] | None
    blurb: str
    title: str
    tags: list[str]
    lore: str
    partype: str
    info: Info
    stats: Stats
    enemy_tips: list[str]
    ally_tips: list[str]
    skins: list[Skin]
    passive: Passive
    spells: list[Spell]



# Quick and simple scraper to pull some data from OPGG using multisearch

# Author  : ShoobyDoo
# Date    : 2023-07-05
# License : BSD-3-Clause


class Stats:
    """
    Represents a player's performance in a game.\n
    
    ### Properties:
        `champion_level: int` - Level of the champion in the game\n
        `damage_self_mitigated: int` - Total damage mitigated by the player\n
        `damage_dealt_to_objectives: int` - Damage dealt to game objectives\n
        `damage_dealt_to_turrets: int` - Damage dealt to turrets\n
        `magic_damage_dealt_player: int` - Magic damage dealt to players\n
        `physical_damage_taken: int` - Physical damage taken by the player\n
        `physical_damage_dealt_to_champions: int` - Physical damage dealt to champions\n
        `total_damage_taken: int` - Total damage taken by the player\n
        `total_damage_dealt: int` - Total damage dealt by the player\n
        `total_damage_dealt_to_champions: int` - Total damage dealt to champions\n
        `largest_critical_strike: int` - Largest critical strike damage\n
        `time_ccing_others: int` - Time spent crowd controlling other players\n
        `vision_score: int` - Vision score of the player\n
        `vision_wards_bought_in_game: int` - Vision wards bought during the game\n
        `sight_wards_bought_in_game: int` - Sight wards bought during the game\n
        `ward_kill: int` - Number of wards killed\n
        `ward_place: int` - Number of wards placed\n
        `turret_kill: int` - Number of turrets killed\n
        `barrack_kill: int` - Number of barracks killed\n
        `kill: int` - Number of kills\n
        `death: int` - Number of deaths\n
        `assist: int` - Number of assists\n
        `largest_multi_kill: int` - Largest multi-kill achieved\n
        `largest_killing_spree: int` - Largest killing spree achieved\n
        `minion_kill: int` - Number of minions killed\n
        `neutral_minion_kill_team_jungle: int` - Neutral minions killed in the team jungle\n
        `neutral_minion_kill_enemy_jungle: int` - Neutral minions killed in the enemy jungle\n
        `neutral_minion_kill: int` - Total neutral minions killed\n
        `gold_earned: int` - Total gold earned by the player\n
        `total_heal: int` - Total amount of healing done by the player\n
        `result: str` - Result of the game (e.g., win, loss)\n
        `op_score: int` - Overall performance score\n
        `op_score_rank: int` - Rank based on overall performance score\n
        `is_opscore_max_in_team: bool` - Whether the player's op score is the highest in the team\n
        `lane_score: int` - Lane performance score\n
        `op_score_timeline: list[dict]` - Timeline of op scores\n
        `op_score_timeline_analysis: dict` - Analysis of op score timeline\n
    """
    def __init__(self,
                 champion_level: int,
                 damage_self_mitigated: int,
                 damage_dealt_to_objectives: int,
                 damage_dealt_to_turrets: int,
                 magic_damage_dealt_player: int,
                 physical_damage_taken: int,
                 physical_damage_dealt_to_champions: int,
                 total_damage_taken: int,
                 total_damage_dealt: int,
                 total_damage_dealt_to_champions: int,
                 largest_critical_strike: int,
                 time_ccing_others: int,
                 vision_score: int,
                 vision_wards_bought_in_game: int,
                 sight_wards_bought_in_game: int,
                 ward_kill: int,
                 ward_place: int,
                 turret_kill: int,
                 barrack_kill: int,
                 kill: int,
                 death: int,
                 assist: int,
                 largest_multi_kill: int,
                 largest_killing_spree: int,
                 minion_kill: int,
                 neutral_minion_kill_team_jungle: int,
                 neutral_minion_kill_enemy_jungle: int,
                 neutral_minion_kill: int,
                 gold_earned: int,
                 total_heal: int,
                 result: str,
                 op_score: int,
                 op_score_rank: int,
                 is_opscore_max_in_team: bool,
                 lane_score: int,
                 op_score_timeline: list[dict],
                 op_score_timeline_analysis: dict) -> None:
        self._champion_level = champion_level
        self._damage_self_mitigated = damage_self_mitigated
        self._damage_dealt_to_objectives = damage_dealt_to_objectives
        self._damage_dealt_to_turrets = damage_dealt_to_turrets
        self._magic_damage_dealt_player = magic_damage_dealt_player
        self._physical_damage_taken = physical_damage_taken
        self._physical_damage_dealt_to_champions = physical_damage_dealt_to_champions
        self._total_damage_taken = total_damage_taken
        self._total_damage_dealt = total_damage_dealt
        self._total_damage_dealt_to_champions = total_damage_dealt_to_champions
        self._largest_critical_strike = largest_critical_strike
        self._time_ccing_others = time_ccing_others
        self._vision_score = vision_score
        self._vision_wards_bought_in_game = vision_wards_bought_in_game
        self._sight_wards_bought_in_game = sight_wards_bought_in_game
        self._ward_kill = ward_kill
        self._ward_place = ward_place
        self._turret_kill = turret_kill
        self._barrack_kill = barrack_kill
        self._kill = kill
        self._death = death
        self._assist = assist
        self._largest_multi_kill = largest_multi_kill
        self._largest_killing_spree = largest_killing_spree
        self._minion_kill = minion_kill
        self._neutral_minion_kill_team_jungle = neutral_minion_kill_team_jungle
        self._neutral_minion_kill_enemy_jungle = neutral_minion_kill_enemy_jungle
        self._neutral_minion_kill = neutral_minion_kill
        self._gold_earned = gold_earned
        self._total_heal = total_heal
        self._result = result
        self._op_score = op_score
        self._op_score_rank = op_score_rank
        self._is_opscore_max_in_team = is_opscore_max_in_team
        self._lane_score = lane_score
        self._op_score_timeline = op_score_timeline
        self._op_score_timeline_analysis = op_score_timeline_analysis

    @property
    def champion_level(self) -> int:
        """
        A `int` representing the champions level
        """
        return self._champion_level
    
    @champion_level.setter
    def champion_level(self, value: int) -> None:
        self._champion_level = value

    @property
    def damage_self_mitigated(self) -> int:
        """
        An `int` representing the amount of damage mitigated on self
        """
        return self._damage_self_mitigated
    
    @damage_self_mitigated.setter
    def damage_self_mitigated(self, value: int) -> None:
        self._damage_self_mitigated = value
        
    @property
    def damage_dealt_to_objectives(self) -> int:
        """
        A `int` representing the amount of damage dealt to objectives overall
        """
        return self._damage_dealt_to_objectives
    
    @damage_dealt_to_objectives.setter
    def damage_dealt_to_objectives(self, value: int) -> None:
        self._damage_dealt_to_objectives = value
    
    @property
    def damage_dealt_to_turrets(self) -> int:
        """
        An `int` representing the amount of damage dealt to turrets specifically
        """
        return self._damage_dealt_to_turrets
    
    @damage_dealt_to_turrets.setter
    def damage_dealt_to_turrets(self, value: int) -> None:
        self._damage_dealt_to_turrets = value
    
    @property
    def magic_damage_dealt_player(self) -> int:
        """
        A `int` representing the amount of magic damage dealt to other players
        """
        return self._magic_damage_dealt_player
    
    @magic_damage_dealt_player.setter
    def magic_damage_dealt_player(self, value: int) -> None:
        self._magic_damage_dealt_player = value
    
    @property
    def physical_damage_taken(self) -> int:
        """
        An `int` representing the amount of physical damage taken
        """
        return self._physical_damage_taken
    
    @physical_damage_taken.setter
    def physical_damage_taken(self, value: int) -> None:
        self._physical_damage_taken = value
    
    @property
    def physical_damage_dealt_to_champions(self) -> int:
        """
        A `int` representing the amount of physical damage dealt to other players
        """
        return self._physical_damage_dealt_to_champions
    
    @physical_damage_dealt_to_champions.setter
    def physical_damage_dealt_to_champions(self, value: int) -> None:
        self._physical_damage_dealt_to_champions = value
    
    @property
    def total_damage_taken(self) -> int:
        """
        An `int` representing the amount of damage taken in total
        """
        return self._total_damage_taken
    
    @total_damage_taken.setter
    def total_damage_taken(self, value: int) -> None:
        self._total_damage_taken = value
        
    @property
    def total_damage_dealt(self) -> int:
        """
        A `int` representing the amount of damage dealt in total
        """
        return self._total_damage_dealt
    
    @total_damage_dealt.setter
    def total_damage_dealt(self, value: int) -> None:
        self._total_damage_dealt = value
        
    @property
    def total_damage_dealt_to_champions(self) -> int:
        """
        An `int` representing the amount of damage dealt to other champions in total
        """
        return self._total_damage_dealt_to_champions
    
    @total_damage_dealt_to_champions.setter
    def total_damage_dealt_to_champions(self, value: int) -> None:
        self._total_damage_dealt_to_champions = value
        
    @property
    def largest_critical_strike(self) -> int:
        """
        A `int` representing the largest critical strike
        """
        return self._largest_critical_strike
    
    @largest_critical_strike.setter
    def largest_critical_strike(self, value: int) -> None:
        self._largest_critical_strike = value
        
    @property
    def time_ccing_others(self) -> int:
        """
        An `int` representing the amount of time cc'ing (crowd controlling) others
        """
        return self._time_ccing_others
    
    @time_ccing_others.setter
    def time_ccing_others(self, value: int) -> None:
        self._time_ccing_others = value
        
    @property
    def vision_score(self) -> int:
        """
        A `int` representing the vision score
        """
        return self._vision_score
    
    @vision_score.setter
    def vision_score(self, value: int) -> None:
        self._vision_score = value
        
    @property
    def vision_wards_bought_in_game(self) -> int:
        """
        An `int` representing the amount of vision wards bought in the game
        """
        return self._vision_wards_bought_in_game
    
    @vision_wards_bought_in_game.setter
    def vision_wards_bought_in_game(self, value: int) -> None:
        self._vision_wards_bought_in_game = value
        
    @property
    def sight_wards_bought_in_game(self) -> int:
        """
        A `int` representing the amount of sight wards bought in the game
        """
        return self._sight_wards_bought_in_game
    
    @sight_wards_bought_in_game.setter
    def sight_wards_bought_in_game(self, value: int) -> None:
        self._sight_wards_bought_in_game = value
        
    @property
    def ward_kill(self) -> int:
        """
        A `int` representing the number of wards killed
        """
        return self._ward_kill
    
    @ward_kill.setter
    def ward_kill(self, value: int) -> None:
        self._ward_kill = value
        
    @property
    def ward_place(self) -> int:
        """
        An `int` representing the number of wards placed
        """
        return self._ward_place
    
    @ward_place.setter
    def ward_place(self, value: int) -> None:
        self._ward_place = value
        
    @property
    def turret_kill(self) -> int:
        """
        An `int` representing the amount of turrets killed
        """
        return self._turret_kill
    
    @turret_kill.setter
    def turret_kill(self, value: int) -> None:
        self._turret_kill = value    
    
    @property
    def barrack_kill(self) -> int:
        """
        An `int` representing the amount of inhibitor kills
        """
        return self._barrack_kill
    
    @barrack_kill.setter
    def barrack_kill(self, value: int) -> None:
        self._barrack_kill = value
    
    @property
    def kill(self) -> int:
        """
        An `int` representing the amount of kills
        """
        return self._kill
    
    @kill.setter
    def kill(self, value: int) -> None:
        self._kill = value
    
    @property
    def death(self) -> int:
        """
        An `int` representing the amount of death
        """
        return self._death
    
    @death.setter
    def death(self, value: int) -> None:
        self._death = value
    
    @property
    def assist(self) -> int:
        """
        An `int` representing the amount of assist
        """
        return self._assist
    
    @assist.setter
    def assist(self, value: int) -> None:
        self._assist = value
    
    @property
    def largest_multi_kill(self) -> int:
        """
        An `int` representing the largest amount of multi-kills
        """
        return self._largest_multi_kill
    
    @largest_multi_kill.setter
    def largest_multi_kill(self, value: int) -> None:
        self._largest_multi_kill = value
    
    @property
    def largest_killing_spree(self) -> int:
        """
        An `int` representing the largest killing spree
        """
        return self._largest_killing_spree
    
    @largest_killing_spree.setter
    def largest_killing_spree(self, value: int) -> None:
        self._largest_killing_spree = value
    
    @property
    def minion_kill(self) -> int:
        """
        An `int` representing the amount of minions killed
        """
        return self._minion_kill
    
    @minion_kill.setter
    def minion_kill(self, value: int) -> None:
        self._minion_kill = value
    
    @property
    def neutral_minion_kill_team_jungle(self) -> int:
        """
        An `int` representing the amount of neutral jungle mobs killed in team jungle
        """
        return self._neutral_minion_kill_team_jungle
    
    @neutral_minion_kill_team_jungle.setter
    def neutral_minion_kill_team_jungle(self, value: int) -> None:
        self._neutral_minion_kill_team_jungle = value
    
    @property
    def neutral_minion_kill_enemy_jungle(self) -> int:
        """
        An `int` representing the amount of neutral jungle mobs killed in enemy jungle
        """
        return self._neutral_minion_kill_enemy_jungle
    
    @neutral_minion_kill_enemy_jungle.setter
    def neutral_minion_kill_enemy_jungle(self, value: int) -> None:
        self._neutral_minion_kill_enemy_jungle = value
    
    @property
    def neutral_minion_kill(self) -> int:
        """
        An `int` representing the amount of neutral minion kills
        """
        return self._neutral_minion_kill
    
    @neutral_minion_kill.setter
    def neutral_minion_kill(self, value: int) -> None:
        self._neutral_minion_kill = value
    
    @property
    def gold_earned(self) -> int:
        """
        An `int` representing the amount of gold earned
        """
        return self._gold_earned
    
    @gold_earned.setter
    def gold_earned(self, value: int) -> None:
        self._gold_earned = value
    
    @property
    def total_heal(self) -> int:
        """
        An `int` representing the total amount healed
        """
        return self._total_heal
    
    @total_heal.setter
    def total_heal(self, value: int) -> None:
        self._total_heal = value
    
    @property
    def result(self) -> str:
        """
        An `str` representing the result of the match (WIN / LOSE)
        """
        return self._result
    
    @result.setter
    def result(self, value: str) -> None:
        self._result = value
    
    @property
    def op_score(self) -> int:
        """
        An `int` representing your op score
        """
        return self._op_score
    
    @op_score.setter
    def op_score(self, value: int) -> None:
        self._op_score = value
    
    @property
    def op_score_rank(self) -> int:
        """
        An `int` representing how your op score stacked up amongst your team (1-10)
        """
        return self._op_score_rank
    
    @op_score_rank.setter
    def op_score_rank(self, value: int) -> None:
        self._op_score_rank = value
    
    @property
    def is_opscore_max_in_team(self) -> bool:
        """
        An `bool` representing whether or not you had the highest op score amongst your team
        """
        return self._is_opscore_max_in_team
    
    @is_opscore_max_in_team.setter
    def is_opscore_max_in_team(self, value: bool) -> None:
        self._is_opscore_max_in_team = value
    
    @property
    def lane_score(self) -> int:
        """
        An `int` representing your lane score
        """
        return self._lane_score
    
    @lane_score.setter
    def lane_score(self, value: int) -> None:
        self._lane_score = value
    
    @property
    def op_score_timeline(self) -> list[dict[str, float]]:
        """
        An `list[dict[str, float]]` representing your op score as the game progresses. Data points are provided every 60 seconds.
        """
        return self._op_score_timeline
    
    @op_score_timeline.setter
    def op_score_timeline(self, value: list[dict[str, float]]) -> None:
        self._op_score_timeline = value
    
    @property
    def op_score_timeline_analysis(self) -> dict[str, str]:
        """
        An `dict[str, str]` representing the timeline analysis
        """
        return self._op_score_timeline_analysis
    
    @op_score_timeline_analysis.setter
    def op_score_timeline_analysis(self, value: dict[str, str]) -> None:
        self._op_score_timeline_analysis = value
    

class GameStats:
    """
    Represents a player's game performance metrics.\n
    
    ### Properties:
        `is_win: bool` - Whether the game was won\n
        `champion_kill: int` - Number of champion kills\n
        `champion_first: bool` - Whether the first champion kill was achieved\n
        `inhibitor_kill: int` - Number of inhibitor kills\n
        `inhibitor_first: bool` - Whether the first inhibitor kill was achieved\n
        `rift_herald_kill: int` - Number of Rift Herald kills\n
        `rift_herald_first: bool` - Whether the first Rift Herald kill was achieved\n
        `dragon_kill: int` - Number of dragon kills\n
        `dragon_first: bool` - Whether the first dragon kill was achieved\n
        `baron_kill: int` - Number of Baron kills\n
        `baron_first: bool` - Whether the first Baron kill was achieved\n
        `tower_kill: int` - Number of tower kills\n
        `tower_first: bool` - Whether the first tower kill was achieved\n
        `horde_kill: int` - Number of horde kills\n
        `horde_first: bool` - Whether the first horde kill was achieved\n
        `is_remake: bool` - Whether the game was a remake\n
        `death: int` - Number of deaths\n
        `assist: int` - Number of assists\n
        `gold_earned: int` - Total gold earned\n
        `kill: int` - Number of kills\n
    """
    def __init__(self,
                 is_win: bool,
                 champion_kill: int,
                 champion_first: bool,
                 inhibitor_kill: int,
                 inhibitor_first: bool,
                 rift_herald_kill: int,
                 rift_herald_first: bool,
                 dragon_kill: int,
                 dragon_first: bool, 
                 baron_kill: int,
                 baron_first: bool, 
                 tower_kill: int,
                 tower_first: bool, 
                 horde_kill: int,
                 horde_first: bool, 
                 is_remake: bool,
                 death: int,
                 assist: int,
                 gold_earned: int,
                 kill: int) -> None:
        self._is_win = is_win
        self._champion_kill = champion_kill
        self._champion_first = champion_first
        self._inhibitor_kill = inhibitor_kill
        self._inhibitor_first = inhibitor_first
        self._rift_herald_kill = rift_herald_kill
        self._rift_herald_first = rift_herald_first
        self._dragon_kill = dragon_kill
        self._dragon_first = dragon_first
        self._baron_kill = baron_kill
        self._baron_first = baron_first
        self._tower_kill = tower_kill
        self._tower_first = tower_first
        self._horde_kill = horde_kill
        self._horde_first = horde_first
        self._is_remake = is_remake
        self._death = death
        self._assist = assist
        self._gold_earned = gold_earned
        self._kill = kill

    @property
    def is_win(self) -> bool:
        """
        A `bool` representing the game result
        """
        return self._is_win
    
    @is_win.setter
    def is_win(self, value: bool) -> None:
        self._is_win = value

    @property
    def champion_kill(self) -> int:
        """
        An `int` representing the number of champions killed
        """
        return self._champion_kill
    
    @champion_kill.setter
    def champion_kill(self, value: int) -> None:
        self._champion_kill = value
        
    @property
    def champion_first(self) -> bool:
        """
        A `bool` representing if team got first champion kill
        """
        return self._champion_first
    
    @champion_first.setter
    def champion_first(self, value: bool) -> None:
        self._champion_first = value
    
    @property
    def inhibitor_kill(self) -> int:
        """
        An `int` representing the number of inhibitors killed
        """
        return self._inhibitor_kill
    
    @inhibitor_kill.setter
    def inhibitor_kill(self, value: int) -> None:
        self._inhibitor_kill = value
    
    @property
    def inhibitor_first(self) -> bool:
        """
        A `bool` representing if team got first inhibitor kill
        """
        return self._inhibitor_first
    
    @inhibitor_first.setter
    def inhibitor_first(self, value: bool) -> None:
        self._inhibitor_first = value
    
    @property
    def rift_herald_kill(self) -> int:
        """
        An `int` representing the number of rift herald kills
        """
        return self._rift_herald_kill
    
    @rift_herald_kill.setter
    def rift_herald_kill(self, value: int) -> None:
        self._rift_herald_kill = value
    
    @property
    def rift_herald_first(self) -> bool:
        """
        A `bool` representing if team got first rift herald kill
        """
        return self._rift_herald_first
    
    @rift_herald_first.setter
    def rift_herald_first(self, value: bool) -> None:
        self._rift_herald_first = value
    
    @property
    def dragon_kill(self) -> int:
        """
        An `int` representing the number of dragon kills
        """
        return self._dragon_kill
    
    @dragon_kill.setter
    def dragon_kill(self, value: int) -> None:
        self._dragon_kill = value
        
    @property
    def dragon_first(self) -> bool:
        """
        A `bool` representing if team got first dragon
        """
        return self._dragon_first
    
    @dragon_first.setter
    def dragon_first(self, value: bool) -> None:
        self._dragon_first = value
        
    @property
    def baron_kill(self) -> int:
        """
        An `int` representing the number of baron kills
        """
        return self._baron_kill
    
    @baron_kill.setter
    def baron_kill(self, value: int) -> None:
        self._baron_kill = value
        
    @property
    def baron_first(self) -> bool:
        """
        A `bool` representing if team got first baron kill
        """
        return self._baron_first
    
    @baron_first.setter
    def baron_first(self, value: bool) -> None:
        self._baron_first = value
        
    @property
    def tower_kill(self) -> int:
        """
        An `int` representing the number of tower kills
        """
        return self._tower_kill
    
    @tower_kill.setter
    def tower_kill(self, value: int) -> None:
        self._tower_kill = value
        
    @property
    def tower_first(self) -> bool:
        """
        A `bool` representing if team got first tower kill
        """
        return self._tower_first
    
    @tower_first.setter
    def tower_first(self, value: bool) -> None:
        self._tower_first = value
        
    @property
    def horde_kill(self) -> int:
        """
        An `int` representing the number of void grub kills
        """
        return self._horde_kill
    
    @horde_kill.setter
    def horde_kill(self, value: int) -> None:
        self._horde_kill = value
        
    @property
    def horde_first(self) -> bool:
        """
        A `bool` representing if team got first void grub kill
        """
        return self._horde_first
    
    @horde_first.setter
    def horde_first(self, value: bool) -> None:
        self._horde_first = value
        
    @property
    def is_remake(self) -> bool:
        """
        A `bool` representing if game was a remake
        """
        return self._is_remake
    
    @is_remake.setter
    def is_remake(self, value: bool) -> None:
        self._is_remake = value
        
    @property
    def death(self) -> int:
        """
        An `int` representing the total amount of deaths for the team
        """
        return self._death
    
    @death.setter
    def death(self, value: int) -> None:
        self._death = value
        
    @property
    def assist(self) -> int:
        """
        An `int` representing the total amount of assists for the team
        """
        return self._assist
    
    @assist.setter
    def assist(self, value: int) -> None:
        self._assist = value    
    
    @property
    def gold_earned(self) -> int:
        """
        An `int` representing the total amount of gold earned for the team
        """
        return self._gold_earned
    
    @gold_earned.setter
    def gold_earned(self, value: int) -> None:
        self._gold_earned = value
    
    @property
    def kill(self) -> int:
        """
        An `int` representing the total amount of kills for the team
        """
        return self._kill
    
    @kill.setter
    def kill(self, value: int) -> None:
        self._kill = value
    

class Team:
    """
    Represents a game's summary including key statistics and banned champions.\n
    
    ### Properties:
        `key: str` - Unique identifier for the game\n
        `game_stat: GameStats` - Detailed statistics of the game\n
        `banned_champions: list` - List of banned champions in the game\n
    """
    def __init__(self,
                 key: str,
                 game_stat: GameStats,
                 banned_champions: list) -> None:
        self._key = key
        self._game_stat = game_stat
        self._banned_champions = banned_champions

    @property
    def key(self) -> str:
        """
        A `str` representing the team key (BLUE, RED, etc.)
        """
        return self._key
    
    @key.setter
    def key(self, value: str) -> None:
        self._key = value
    
    @property
    def game_stat(self) -> GameStats:
        """
        A `GameStats` object representing all game specific stats
        """
        return self._game_stat
    
    @game_stat.setter
    def game_stat(self, value: GameStats) -> None:
        self._game_stat = value
    
    @property
    def banned_champions(self) -> list:
        """
        A `list` of champion ids that were banned
        """
        return self._banned_champions
    
    @banned_champions.setter
    def banned_champions(self, value: list) -> None:
        self._banned_champions = value



import os
import sqlite3
import logging

from opgg.v2.champion import Champion
from opgg.v2.search_result import SearchResult


class Cacher:
    """
    Cacher class for caching summoners, champions, and seasons.

    ### Properties:
        `db_path` - Path to the database file.\n
        `logger` - Logger instance.
    """

    def __init__(self, db_path=f"./cache/opgg.py.db"):
        self.db_path = db_path
        self.logger = logging.getLogger("OPGG.py")

    def setup(self) -> None:
        """
        Sets up the cache database and table(s).

        Runs at OPGG object creation.
        """
        if not os.path.exists("./cache"):
            self.logger.info("Creating cache directory...")
            os.mkdir("./cache")
            self.logger.info("Setting up cache database...")

        self.conn = sqlite3.connect(self.db_path)
        self.cursor = self.conn.cursor()

        # Create summoner table if it doesn't exist
        self.logger.debug("Creating summoner table if it doesn't exist...")
        self.cursor.execute(
            """
            CREATE TABLE IF NOT EXISTS tblSummoners (
                summoner_id PRIMARY KEY,
                game_name,
                tagline
            );
            """
        )

        # Create index on game_name
        self.cursor.execute(
            """
            CREATE INDEX IF NOT EXISTS idx_game_name ON tblSummoners (game_name);
            """
        )

        # Create composite index on game_name and tagline
        self.cursor.execute(
            """
            CREATE INDEX IF NOT EXISTS idx_game_name_tagline ON tblSummoners (game_name, tagline);
            """
        )

        self.logger.debug("Creating champions table if it doesn't exist...")
        self.cursor.execute(
            """
            CREATE TABLE IF NOT EXISTS tblChampions (
                champion_id PRIMARY KEY,
                name
            );
            """
        )

        self.conn.commit()
        self.conn.close()

    def get_champ_id_by_name(self, champ_name: str) -> int | None:
        """
        Get the ID of a champion by name.

        Args:
            `champ_name` (`str`): The name of the champion

        Returns:
            `int | None`: The ID of the champion, or None if not found
        """
        self.conn = sqlite3.connect(self.db_path)
        self.cursor = self.conn.cursor()

        self.cursor.execute(
            "SELECT * FROM tblChampions WHERE name LIKE ?",
            (f"%{champ_name}%",),
        )
        result = self.cursor.fetchone()

        self.conn.close()

        # if the result is a tuple, return the first element (-> champ_id, name), otherwise return None
        return result[0] if isinstance(result, tuple) else None

    def cache_champs(self, champs: list[Champion]) -> None:
        """
        Cache champions in the database.

        Args:
            `champs` (`list[Champion]`): List of champions to cache
        """
        self.conn = sqlite3.connect(self.db_path)
        self.cursor = self.conn.cursor()

        # use executemany to bulk insert
        self.logger.debug(
            f"Attempting to insert {len(champs)} champions into cache database..."
        )
        self.cursor.executemany(
            """
            INSERT OR IGNORE INTO tblChampions (champion_id, name)
            VALUES (?, ?);
            """,
            [(champ.id, champ.name) for champ in champs],
        )

        self.conn.commit()
        self.conn.close()

        if len(champs) == self.get_cached_champs_count():
            self.logger.debug(
                f"Successfully inserted {len(champs)} champions into cache database."
            )
        else:
            self.logger.warning(
                f"Failed to insert {len(champs) - self.get_cached_champs_count()} champions into cache database."
            )
            self.logger.debug("CHAMPS DUMP: ", champs)

    def get_cached_champs_count(self) -> int:
        """
        Get the count of cached champions.

        Returns:
            `int`: The count of cached champions.
        """
        self.conn = sqlite3.connect(self.db_path)
        self.cursor = self.conn.cursor()

        self.cursor.execute("SELECT COUNT(*) FROM tblChampions")
        count = self.cursor.fetchone()[0]

        self.conn.close()

        self.logger.debug(f"Found {count} cached champions.")

        return count

    def cache_search_results(self, search_results: list[SearchResult]) -> None:
        """
        Cache search results in the database.

        Args:
            `search_results` (`list[SearchResult]`): List of search results to cache
        """
        self.conn = sqlite3.connect(self.db_path)
        self.cursor = self.conn.cursor()

        try:
            for result in search_results:
                summoner = result.summoner
                self.logger.debug(
                    f"Processing summoner for caching: {summoner.game_name}#{summoner.tagline}"
                )

                # Check if summoner exists
                self.cursor.execute(
                    """
                    SELECT summoner_id FROM tblSummoners 
                    WHERE summoner_id = ?
                    """,
                    (summoner.summoner_id,),
                )

                exists = self.cursor.fetchone()

                if exists:
                    # Update existing summoner if game_name or tagline changed
                    self.cursor.execute(
                        """
                        UPDATE tblSummoners 
                        SET game_name = ?, tagline = ?
                        WHERE summoner_id = ? 
                        AND (game_name != ? OR tagline != ?)
                        """,
                        (
                            summoner.game_name,
                            summoner.tagline,
                            summoner.summoner_id,
                            summoner.game_name,
                            summoner.tagline,
                        ),
                    )
                else:
                    # Insert new summoner
                    self.cursor.execute(
                        """
                        INSERT INTO tblSummoners (summoner_id, game_name, tagline)
                        VALUES (?, ?, ?)
                        """,
                        (summoner.summoner_id, summoner.game_name, summoner.tagline),
                    )

            self.conn.commit()
            self.logger.info(
                f"Successfully cached {len(search_results)} search results"
            )

        except Exception as e:
            self.logger.error(f"Error caching search results: {str(e)}")
            self.conn.rollback()
        finally:
            self.conn.close()

    def is_search_result_cached(
        self, game_name: str, tagline: str = ""
    ) -> tuple[bool, str | None]:
        """
        Check if a search result exists in the cache.

        Args:
            `game_name` (`str`): The summoner's game name
            `tagline` (`str`, optional): The summoner's tagline. Defaults to empty string.

        Returns:
            `tuple[bool, str | None]`: Tuple containing:
                - `bool`: Whether the summoner is cached
                - `str | None`: The summoner_id if cached, None if not
        """
        self.conn = sqlite3.connect(self.db_path)
        self.cursor = self.conn.cursor()

        try:
            if tagline:
                # Search with both game_name and tagline
                self.cursor.execute(
                    """
                    SELECT summoner_id FROM tblSummoners 
                    WHERE game_name = ? AND tagline = ?
                    """,
                    (game_name, tagline),
                )
            else:
                # Search with just game_name
                self.cursor.execute(
                    """
                    SELECT summoner_id FROM tblSummoners 
                    WHERE game_name = ?
                    """,
                    (game_name,),
                )

            result = self.cursor.fetchone()
            return (True, result[0]) if result else (False, None)

        except Exception as e:
            self.logger.error(f"Error checking cache: {str(e)}")
            return False, None
        finally:
            self.conn.close()



from datetime import datetime
from typing import Optional
from pydantic import BaseModel

from opgg.v2.game import LiveGame


class UpdateData(BaseModel):
    """Data returned from an update request."""

    message: str
    """Status message from the update operation."""

    last_updated_at: datetime
    """When the profile was last updated."""

    renewable_at: Optional[datetime] = None
    """When the profile can be updated again."""

    finish: Optional[bool] = None
    """Whether the update operation completed."""

    delay: Optional[int] = None
    """Delay in seconds before next allowed update."""

    class Config:
        """Pydantic model configuration."""

        json_encoders = {datetime: lambda v: v.isoformat()}


class UpdateResponse(BaseModel):
    """Response from an update request."""

    status: int
    """HTTP status code of the response."""

    data: UpdateData
    """The update operation data."""


class LiveGameResponse(BaseModel):
    """Response from a live game request."""

    data: Optional[LiveGame] = None
    """The live game data, if available."""

    status: int
    """HTTP status code of the response."""

    message: str
    """Status message from the request."""

    detail: Optional[str] = None
    """Additional details about the request."""



# Simple scraper to pull some data from OPGG using multisearch

# Author  : ShoobyDoo
# Date    : 2023-07-05
# License : BSD-3-Clause


class Region:
    """
    Struct for regions.
    
    ### Options:
        `NA` - North America\n
        `EUW` - Europe West\n
        `EUNE` - Europe Nordic & East\n
        `KR` - Korea\n
        `JP` - Japan\n
        `BR` - Brazil\n
        `LAN` - Latin America North\n
        `LAS` - Latin America South\n
        `OCE` - Oceania\n
        `RU` - Russia\n
        `TR` - Turkey
    """
    
    NA = "NA"
    EUW = "EUW"
    EUNE = "EUNE"
    KR = "KR"
    JP = "JP"
    BR = "BR"
    LAN = "LAN"
    LAS = "LAS"
    OCE = "OCE"
    RU = "RU"
    TR = "TR"


class By:
    """
    Struct for search-by or match-by types.
    
    ### Options:
        `ID` - Generic ID\n
        `KEY` - Generic Key\n
        `NAME` - Generic Name\n
        `COST` - Generic Cost\n
        `BLUE_ESSENCE` - Specific Cost (Blue Essence)\n
        `RIOT_POINTS` - Specific Cost (Riot Points)
    """
        
    ID = "id"
    KEY = "key"
    NAME = "name"
    COST = "cost"
    BLUE_ESSENCE = "BE"
    RIOT_POINTS = "RP"
    

class Queue:
    """
    Struct for queue types.
    
    ### Options:
        `SOLO` - SoloQueue\n
        `FLEX` - FlexQueue\n
        `ARENA` - Arena
    """
    
    SOLO = "SOLORANKED"
    FLEX = "FLEXRANKED"
    ARENA = "ARENA"
    


# Quick and simple scraper to pull some data from OPGG using multisearch

# Author  : ShoobyDoo
# Date    : 2023-07-05
# License : BSD-3-Clause

from datetime import datetime


class Tier:
    """
    Represents a tier in a league.\n
    
    ### Properties:
        `tier: str` - Tier name\n
        `division: int` - Division number\n
        `lp: int` - League points. Defaults to 0 if None\n
        `level: int` - Unknown level, usually is None/Null. Defaults to 0 if None\n
        `tier_image_url: str` - URL to the tier image. Defaults to None\n
        `border_image_url: str` - URL to the border image. Defaults to None\n
    """
    def __init__(self,
                 tier: str,
                 division: int,
                 tier_image_url: str = None,
                 border_image_url: str = None,
                 lp: int = None,
                 level: int = None) -> None:
        self._tier = tier if tier != None else "UNRANKED"
        self._division = division if division != None else 0
        self._lp = lp if lp != None else 0
        self._level = level if level != None else 0
        self._tier_image_url = tier_image_url
        self._border_image_url = border_image_url
        
    @property
    def tier(self) -> str:
        """
        A `str` representing the tier (e.g. "Silver")
        """
        return self._tier
    
    @tier.setter
    def tier(self, value: str) -> None:
        self._tier = value
    
    @property
    def division(self) -> int:
        """
        An `int` representing the division
        """
        return self._division
    
    @division.setter
    def division(self, value: int) -> None:
        self._division = value
        
    @property
    def lp(self) -> int:
        """
        An `int` representing the league points
        """
        return self._lp
    
    @lp.setter
    def lp(self, value: int) -> None:
        self._lp = value
    
    @property
    def level(self) -> int:
        """
        An `int` representing an unknown level value
        """
        return self._level
    
    @level.setter
    def level(self, value: int) -> None:
        self._level = value
        
    @property
    def tier_image_url(self) -> str:
        """
        A `str` representing the URL to the tier image
        """
        return self._tier_image_url
    
    @tier_image_url.setter
    def tier_image_url(self, value: str) -> None:
        self._tier_image_url = value
        
    @property
    def border_image_url(self) -> str:
        """
        A `str` representing the URL to the border image
        """
        return self._border_image_url 
     
    @border_image_url.setter 
    def border_image_url(self, value: str) -> None:
        self._border_image_url = value

    def __repr__(self) -> str:
        return f"Tier(tier={self._tier}, division={self._division}, lp={self._lp})"
    

class QueueInfo:
    """
    Represents a queue in a league.\n
    
    ### Properties:
        `id: int` - Queue ID\n
        `queue_translate: str` - Game type in KOREAN\n
        `game_type: str` - Queue/Game type\n
    """
    def __init__(self,
                 id: int,
                 queue_translate: str,
                 game_type: str) -> None:
        self._id = id
        self._queue_translate = queue_translate
        self._game_type = game_type
        
    @property
    def id(self) -> int:
        """
        An `int` representing the queue ID
        """
        return self._id
     
    @id.setter
    def id(self, value: int) -> None:
        self._id = value
        
    @property
    def queue_translate(self) -> str:
        """
        A `str` representing the queue/game type in KOREAN
        """
        return self._queue_translate
    
    @queue_translate.setter
    def queue_translate(self, value: str) -> None:
        self._queue_translate = value
        
    @property
    def game_type(self) -> str:
        """
        A `str` representing the queue/game type
        """
        return self._game_type
    
    @game_type.setter
    def game_type(self, value: str) -> None:
        self._game_type = value
    
    def __repr__(self) -> str:
        return f"QueueInfo(game_type={self.game_type})"


class LeagueStats:
    """
    Represents a user's league stats.\n
    
    ### Properties:
        `queue_info: QueueInfo` - QueueInfo object\n
        `tier_info: Tier` - Tier object\n
        `win: int` - Number of wins. Defaults to 0 if None\n
        `lose: int` - Number of losses. Defaults to 0 if None\n
        `is_hot_streak: bool` - Whether or not the user is on a hot streak (3+ win streak)\n
        `is_fresh_blood: bool` - Whether or not the user is fresh blood\n
        `is_veteran: bool` - Whether or not the user is a veteran\n
        `is_inactive: bool` - Whether or not the user is inactive\n
        `series: bool` - Series object\n
        `updated_at: datetime` - Datetime object representing the last time the stats were updated\n
    """
    def __init__(self,
                 queue_info: QueueInfo,
                 tier_info: Tier,
                 win: int,
                 lose: int,
                 is_hot_streak: bool,
                 is_fresh_blood: bool,
                 is_veteran: bool,
                 is_inactive: bool,
                 series: bool,
                 updated_at: datetime) -> None:
        self._queue_info = queue_info
        self._tier_info = tier_info
        self._win = win if win != None else 0
        self._lose = lose if lose != None else 0
        self._is_hot_streak = is_hot_streak
        self._is_fresh_blood = is_fresh_blood 
        self._is_veteran = is_veteran 
        self._is_inactive = is_inactive 
        self._series = series 
        self._updated_at = updated_at 
    
    @property
    def queue_info(self) -> QueueInfo:
        """
        A `QueueInfo` object representing the queue
        """
        return self._queue_info
    
    @queue_info.setter
    def queue_info(self, value: QueueInfo) -> None:
        self._queue_info = value
        
    @property
    def tier_info(self) -> Tier:
        """
        A `Tier` object representing the tier
        """
        return self._tier_info
    
    @tier_info.setter
    def tier_info(self, value: Tier) -> None: 
        self._tier_info = value
        
    @property
    def win(self) -> int:
        """
        An `int` representing the number of wins
        """
        return self._win
    
    @win.setter
    def win(self, value: int) -> None:
        self._win = value
        
    @property
    def lose(self) -> int:
        """
        An `int` representing the number of losses
        """
        return self._lose
    
    @lose.setter
    def lose(self, value: int) -> None:
        self._lose = value
        
    @property
    def is_hot_streak(self) -> bool:
        """
        A `bool` representing whether or not the user is on a hot streak (3+ winstreak)
        """
        return self._is_hot_streak
    
    @is_hot_streak.setter
    def is_hot_streak(self, value: bool) -> None:
        self._is_hot_streak = value
        
    @property
    def is_fresh_blood(self) -> bool:
        """
        A `bool` representing whether or not the user is fresh blood
        """
        return self._is_fresh_blood
    
    @is_fresh_blood.setter
    def is_fresh_blood(self, value: bool) -> None:
        self._is_fresh_blood = value
        
    @property
    def is_veteran(self) -> bool:
        """
        A `bool` representing whether or not the user is a veteran
        """
        return self._is_veteran
    
    @is_veteran.setter
    def is_veteran(self, value: bool) -> None:
        self._is_veteran = value
        
    @property
    def is_inactive(self) -> bool:
        """
        A `bool` representing whether or not the user is inactive
        """
        return self._is_inactive
     
    @is_inactive.setter
    def is_inactive(self, value: bool) -> None:
        self._is_inactive = value
        
    @property
    def series(self) -> bool:
        """
        A `bool` representing if the summoner is in a series
        """
        return self._series
     
    @series.setter
    def series(self, value) -> None:
        self._series = value
        
    @property
    def updated_at(self) -> datetime:
        """
        A `datetime` object representing the last time the stats were updated
        """
        return self._updated_at
    
    @updated_at.setter
    def updated_at(self, value: datetime) -> None:
        self._updated_at = value
    
    @property
    def win_rate(self) -> float:
        """
        A `float` representing the win rate of the champion.
        """
        if self._win and self._lose:
            return round(float((self._win / (self._win + self._lose)) * 100), 2) if (self._win + self._lose) != 0 else 0
        return 0
    
    
    def __repr__(self) -> str:
        return f"LeagueStats(queue_info={self.queue_info}, tier_info={self.tier_info}, win={self.win} / lose={self.lose} (winrate: {self.win_rate}%))"


import sqlite3
import logging
import os
import glob
from datetime import datetime

from opgg.v1.champion import Champion, Passive, Skin, Spell
from opgg.v1.season import SeasonInfo


class Cacher:
    """
    Cacher class for caching summoners, champions, and seasons.
    
    ### Properties:
        `db_path` - Path to the database file.\n
        `logger` - Logger instance.
    """
    def __init__(self, db_path = f'./cache/opgg-{datetime.now().strftime("%Y-%m-%d")}.db'):
        self.db_path = db_path
        self.logger = logging.getLogger("OPGG.py")
    
    
    def setup(self) -> None:
        """
        Sets up the cache database and table(s). 
        
        Runs at OPGG object creation.
        """
        if not os.path.exists('./cache'):
            self.logger.info("Creating cache directory...")
            os.mkdir('./cache')
            self.logger.info("Setting up cache database...")
        
        # Do all this cleanup and verification in the connect function to minimize work required
        # to enforce this. Anytime a function needs to connect to the database, this check should
        # be performed and the data base structure should be verified. 
        # 
        # It should also then query out to pull the necessary data as if this is called on a 
        # get_<whatever> type function the database will have no data following a cache rebuild.
        cache_db = glob.glob("./cache/opgg*.db")
        
        if (len(cache_db) > 0 and "-" in cache_db[0]):
            old_path = cache_db[0]
            cache_last_updated = datetime.strptime(old_path.split("-", 1)[-1].replace(".db", ""), "%Y-%m-%d")
            
            self.logger.info(f"Cache found! Was last built: {cache_last_updated}")
            
            if (datetime.now() - cache_last_updated).days >= 7:
                self.logger.info("Cache is older than 1 week, rebuilding...")
                
                # set db_path to the old cache for a second
                new_path = self.db_path
                
                self.logger.info("Deleting old cache data...")
                os.remove(old_path)
                
                self.logger.info(f"Updating filename with current date {old_path} -> {new_path}")
                self.db_path = new_path
                
                self.logger.info("Cache has been removed! The immediate request following a cache rebuild might take slightly longer as new data is fetched and the cache is updated.")
                
        elif (len(cache_db) > 0):
            os.remove(cache_db[0])
        
        self.conn = self.connect()
        self.cursor = self.conn.cursor()
        
        # Create summoner table if it doesn't exist
        self.logger.debug("Creating summoner table if it doesn't exist...")
        self.cursor.execute("""CREATE TABLE IF NOT EXISTS tblSummoners (summoner_name PRIMARY KEY, summoner_id);""")
        
        # Create champions table if it doesn't exist
        self.logger.debug("Creating champions table if it doesn't exist...")
        self.cursor.execute(
            """
            CREATE TABLE IF NOT EXISTS tblChampions (
                champion_id PRIMARY KEY,
                champion_key,
                champion_name,
                champion_image_url,
                champion_evolve_list,
                champion_partype
            );
            """
        )
        
        # Create seasons table if it doesn't exist
        self.logger.debug("Creating seasons table if it doesn't exist...")
        self.cursor.execute(
            """
            CREATE TABLE IF NOT EXISTS tblSeasonInfo (
                season_id PRIMARY KEY,
                season_value,
                season_display_name,
                season_split,
                season_is_preseason
            );
            """
        )
        
        # Create passives table if it doesn't exist
        self.logger.debug("Creating passives table if it doesn't exist...")
        self.cursor.execute(
            """
            CREATE TABLE IF NOT EXISTS tblPassives (
                champion_id PRIMARY KEY,
                passive_name,
                passive_description,
                passive_image_url,
                passive_video_url
            );
            """
        )
        
        # Create spells table if it doesn't exist
        self.logger.debug("Creating spells table if it doesn't exist...")
        self.cursor.execute(
            """
            CREATE TABLE IF NOT EXISTS tblSpells (
                champion_id,
                spell_key,
                spell_name PRIMARY KEY,
                spell_description,
                spell_max_rank,
                spell_range_burn_list,
                spell_cooldown_burn_list,
                spell_cooldown_burn_float_list,
                spell_cost_burn_list,
                spell_tooltip,
                spell_image_url,
                spell_video_url
            );
            """
        )
        
        # Create skins table if it doesn't exist
        self.logger.debug("Creating skins table if it doesn't exist...")
        self.cursor.execute(
            """
            CREATE TABLE IF NOT EXISTS tblSkins (
                champion_id,
                skin_id PRIMARY KEY,
                skin_name,
                skin_centered_image,
                skin_video_url,
                skin_prices,
                skin_sales,
                skin_release_date
            )
            """
        )
        
        self.conn.commit()
        self.conn.close()
    
    
    def insert_summoner(self, summoner_name: str, summoner_id: str, return_result: bool = False) -> None | str:
        """
        Inserts a summoner name and id into the database.
        
        ### Args:
            summoner_name : `str`
                Summoner name.
                
            summoner_id : `str`
                Summoner ID.
        
        ### Returns:
            `str, optional` : Returns a string with the amount of rows affected if requested.
        """
        self.conn = self.connect()
        self.cursor = self.conn.cursor()
        
        self.logger.debug(f"Attempting to insert {summoner_name} into cache database...")
        
        self.cursor.execute(
            """
            INSERT OR IGNORE INTO tblSummoners (summoner_name, summoner_id)
            VALUES (?, ?);
            """, 
            (summoner_name, summoner_id)
        )
        
        self.conn.commit()
        self.conn.close()
        
        return_msg = f"You have made changes to the database. Table: tblSummoners | Rows affected: {self.cursor.rowcount}"
        
        if return_result:
            return return_msg
        else:
            self.logger.info(return_msg)
        
        
    def get_summoner_id(self, summoner_name: str) -> str | None:
        """
        Gets a summoner id from the cache database by a provided summoner name.
        
        `Note: I'm now realizing there are quite a few limitations to this method.`\n
        `There is currently no way to specify region in the search, different regions could have the same summoner name.`
        `This method's search logic will need to be reworked.`
        
        ### Args:
            summoner_name : `str`
                Summoner name.
        
        ### Returns:
            `str | None` : Returns a `str` with the summoner id, if found. Otherwise returns `None`.
        """
        self.conn = self.connect()
        self.cursor = self.conn.cursor()
        
        self.logger.info(f"Getting {summoner_name}'s summoner id from cache database...")
        
        self.cursor.execute("""
            SELECT summoner_id
            FROM tblSummoners
            WHERE summoner_name = ?;
        """, (summoner_name,))
        
        result = self.cursor.fetchone()
        self.conn.close()
        
        if result is None:
            self.logger.info(f"{summoner_name}'s summoner_id not found in cache database.")
            return None
        
        self.logger.info(f"{summoner_name}'s summoner_id found in cache database. ({result[0]})")
        return result[0]
    
    
    def get_summoner_name(self, summoner_id: str) -> str | None:
        """
        Gets a summoner name from the cache database by a provided summoner id.
        
        ### Args:
            summoner_id : `str`
                Summoner ID.
        
        ### Returns:
            `str | None` : Returns a `str` with the summoner name, if found. Otherwise returns `None`.
        """
        self.conn = self.connect()
        self.cursor = self.conn.cursor()
        
        self.logger.info(f"Getting associated summoner name from summoner_id: {summoner_id}...")
        
        self.cursor.execute("""
            SELECT summoner_name
            FROM tblSummoners
            WHERE summoner_id = ?;
        """, (summoner_id,))
        
        result = self.cursor.fetchone()
        self.conn.close()
        
        if result is None:
            self.logger.info(f"Could not find an associated summoner_name for summoner_id: {summoner_id}")
            return None
        
        self.logger.info(f"Found associated summoner_name for summoner_id: {summoner_id} ({result[0]})")
        return result[0]
    
    
    def insert_all_champs(self, champions: list[Champion], return_result: bool = False) -> None | str:
        """
        Inserts a list of champions and their related attributes into the database.
        
        ### Args:
            champions : `list[Champion]`
                A list of Champion objects to be cached.
        
        ### Returns:
            `None` | `str, optional` : Returns a string with the amount of rows affected if requested.
        """
        self.conn = self.connect()
        self.cursor = self.conn.cursor()
        total_rc = 0 # total rowcount
        return_msg = "You've made changes to the database. Table: {table} | Rows affected: {count}"
        
        self.logger.debug(f"Attempting to insert {len(champions)} champions into cache database...")
        
        batch_champion_insert:  list[tuple] = []
        batch_passives_insert:  list[tuple] = []
        batch_spells_insert:    list[tuple] = []
        batch_skins_insert:     list[tuple] = []
        
        for champion in champions:
            # champion
            batch_champion_insert.append((
                champion.id,
                champion.key,
                champion.name,
                champion.image_url,
                ','.join([f"{_evolve}" for _evolve in champion.evolve]),
                champion.partype
            ))
            
            # passive (champ id to map passive to champion)
            batch_passives_insert.append((
                champion.id,
                champion.passive.name,
                champion.passive.description,
                champion.passive.image_url,
                champion.passive.video_url
            ))
            
            # spells (champ id to map spell to champion)
            for spell in champion.spells:
                batch_spells_insert.append((
                    champion.id,
                    spell.key,
                    spell.name,
                    spell.description,
                    spell.max_rank,
                    ','.join([f"{_range}" for _range in spell.range_burn]),
                    ','.join([f"{_cooldown}" for _cooldown in spell.cooldown_burn]),
                    ','.join([f"{_cooldown_float}" for _cooldown_float in spell.cooldown_burn_float]),
                    ','.join([f"{_cost}" for _cost in spell.cost_burn]),
                    spell.tooltip,
                    spell.image_url,
                    spell.video_url
                ))
            
            # skins (champ id to map skin to champion)
            for skin in champion.skins:
                batch_skins_insert.append((
                    champion.id,
                    skin.id,
                    skin.name,
                    skin.centered_image,
                    skin.skin_video_url,
                    ','.join([f"{price.currency}: {price.cost}" for price in skin.prices]) if skin.prices else None,
                    skin.release_date,
                    ','.join([f"{_price}" for _price in skin.prices]) if skin.prices else str(skin.prices)
                ))
        
        # insert into champion table
        self.cursor.executemany(
            """
            INSERT OR IGNORE INTO tblChampions (champion_id, champion_key, champion_name, champion_image_url, champion_evolve_list, champion_partype)
            VALUES (:1, :2, :3, :4, :5, :6)
            """,
            batch_champion_insert
        )
        
        total_rc += self.cursor.rowcount
        self.logger.debug(return_msg.format(table="tblChampions", count=self.cursor.rowcount))
        
        # insert into passives table
        self.cursor.executemany(
            """
            INSERT OR IGNORE INTO tblPassives (champion_id, passive_name, passive_description, passive_image_url, passive_video_url)
            VALUES (:1, :2, :3, :4, :5)
            """,
            batch_passives_insert
        )
        
        total_rc += self.cursor.rowcount
        self.logger.debug(return_msg.format(table="tblPassives", count=self.cursor.rowcount))
        
        # insert into skins table
        self.cursor.executemany(
            """
            INSERT OR IGNORE INTO tblSkins (champion_id, skin_id, skin_name, skin_centered_image, skin_video_url, skin_prices, skin_release_date, skin_sales)
            VALUES (:1, :2, :3, :4, :5, :6, :7, :8)
            """,
            batch_skins_insert
        )
        
        total_rc += self.cursor.rowcount
        self.logger.debug(return_msg.format(table="tblSkins", count=self.cursor.rowcount))
        
        # insert into spells table
        self.cursor.executemany(
            """
            INSERT OR IGNORE INTO tblSpells (champion_id, spell_key, spell_name, spell_description, spell_max_rank, spell_range_burn_list, spell_cooldown_burn_list, spell_cooldown_burn_float_list, spell_cost_burn_list, spell_tooltip, spell_image_url, spell_video_url)
            VALUES (:1, :2, :3, :4, :5, :6, :7, :8, :9, :10, :11, :12)
            """,
            batch_spells_insert
        )
        
        total_rc += self.cursor.rowcount
        self.logger.debug(return_msg.format(table="tblSpells", count=self.cursor.rowcount))
        
        self.conn.commit()
        self.conn.close()
        
        return_msg = f"You've made several changes to the database. Total rows affected: {total_rc}"
        
        if return_result: 
            return return_msg
        else:
            self.logger.info(return_msg)
            
            
    def get_all_champs(self) -> list[Champion] | None:
        """
        Gets all champions from the cache database and returns a list of Champion objects.

        ### Returns:
            `list[Champion]` | `None` : Returns a list of Champion objects if found. Otherwise returns `None`.
        """
        self.conn = self.connect()
        self.cursor = self.conn.cursor()
        all_champs = []
        
        self.logger.info("Getting all champions from cache database...")
        self.cursor.execute("SELECT * FROM tblChampions;")
        result = self.cursor.fetchall()
        
        if result is None:
            self.logger.error("No champions found in cache database.")
            raise Exception("Multiple levels of data fetching have failed to result in this event.\n \
                            1. There were no champions returned in the request to opgg (?) See debug logs...\n \
                            2. There were no champions found in the cache database.")
        else:
            self.logger.info(f"Found {len(result)} champions in cache database.")
            cached_champ: tuple[str, str, str, str, str]
            for i, cached_champ in enumerate(result):
                # In order to restore a champion object, we need the following:
                # PASSIVE FROM PASSIVES TABLE
                # SPELLS FROM SPELLS TABLE
                # SKINS FROM SKINS TABLE
                champ_passive = self.get_passive(cached_champ[0])
                champ_spells = self.get_spells(cached_champ[0])
                champ_skins = self.get_skins(cached_champ[0])
                
                champ_obj = Champion(
                    id=cached_champ[0],
                    key=cached_champ[1],
                    name=cached_champ[2],
                    image_url=cached_champ[3],
                    evolve=cached_champ[4].split(',') if cached_champ[4] else None,
                    partype=cached_champ[5],
                    passive=champ_passive,
                    spells=champ_spells,
                    skins=champ_skins
                )
                all_champs.append(champ_obj)
                self.logger.info(f"Successfully rebuilt the \"{champ_obj.name}\" champion object from cache. ({i+1}/{len(result)})")
                
            return all_champs
                
            
    def get_passive(self, champion_id: int) -> Passive | None:
        """
        Gets a champion's passive from the cache database.
        
        ### Args:
            champion_id : `int`
                Champion ID.
        
        ### Returns:
            `Passive | None` : Returns a `Passive` object if found. Otherwise returns `None`.
        """
        self.conn = self.connect()
        self.cursor = self.conn.cursor()
        
        self.logger.debug(f"Getting passive for champion_id: {champion_id}...")
        
        self.cursor.execute(
            """
            SELECT *
            FROM tblPassives
            WHERE champion_id = ?;
            """, (champion_id,)
        )
        
        result = self.cursor.fetchone()
        self.conn.close()
        
        if result is None:
            self.logger.debug(f"Passive not found for champion_id: {champion_id}.")
            return None
        
        self.logger.debug(f"Passive \"{result[1]}\" found for champion_id: {champion_id}.")
        return Passive(
            name=result[1],
            description=result[2],
            image_url=result[3],
            video_url=result[4]
        )    
         
            
    def get_spells(self, champion_id: int) -> list[Spell] | None:
        """
        Gets a champion's spells from the cache database.
        
        ### Args:
            champion_id : `int`
                Champion ID.
        
        ### Returns:
            `list[Spell] | None` : Returns a list of `Spell` objects if found. Otherwise returns `None`.
        """
        self.conn = self.connect()
        self.cursor = self.conn.cursor()
        
        self.logger.debug(f"Getting spells for champion_id: {champion_id}...")
        
        self.cursor.execute(
            """
            SELECT *
            FROM tblSpells
            WHERE champion_id = ?;
            """, (champion_id,)
        )
        
        result = self.cursor.fetchall()
        self.conn.close()
        
        if result is None:
            self.logger.debug(f"No spells found for champion_id: {champion_id}.")
            return None
        
        self.logger.debug(f"Found spells for champion_id: {champion_id}.")
        return [Spell(
            key=spell[1],
            name=spell[2],
            description=spell[3],
            max_rank=spell[4],
            range_burn=spell[5].split(',') if spell[5] else None,
            cooldown_burn=spell[6].split(',') if spell[6] else None,
            cooldown_burn_float=spell[7].split(',') if spell[7] else None,
            cost_burn=spell[8].split(',') if spell[8] else None,
            tooltip=spell[9],
            image_url=spell[10],
            video_url=spell[11]
        ) for spell in result]
    
    
    def get_skins(self, champion_id: int) -> list[Skin] | None:
        """
        Gets a champion's skins from the cache database.
        
        ### Args:
            champion_id : `int`
                Champion ID.
        
        ### Returns:
            `list[Skin] | None` : Returns a list of `Skin` objects if found. Otherwise returns `None`.
        """
        self.conn = self.connect()
        self.cursor = self.conn.cursor()
        
        self.logger.debug(f"Getting skins for champion_id: {champion_id}...")
        
        self.cursor.execute(
            """
            SELECT *
            FROM tblSkins
            WHERE champion_id = ?;
            """, (champion_id,)
        )
        
        result = self.cursor.fetchall()
        self.conn.close()
        
        if result is None:
            self.logger.debug(f"No skins found for champion_id: {champion_id}.")
            return None
        
        self.logger.debug(f"Found skins for champion_id: {champion_id}.")
        return [Skin(
            champion_id=skin[0],
            id=skin[1],
            name=skin[2],
            centered_image=skin[3],
            skin_video_url=skin[4],
            prices=skin[5].split(',') if skin[5] else None,
            release_date=skin[6]
        ) for skin in result] 
    
    
    def insert_all_seasons(self, seasons: list[SeasonInfo], return_result: bool = False) -> None | str:
        """
        Inserts a list of seasons and their related attributes into the database.
        
        ### Args:
            seasons : `list[SeasonInfo]`
                A list of SeasonInfo objects to be cached.
        
        ### Returns:
            `None` | `str, optional` : Returns a string with the amount of rows affected if requested.
        """
        self.conn = self.connect()
        self.cursor = self.conn.cursor()
        total_rc = 0
        return_msg = "You've made changes to the database. Table: {table} | Rows affected: {count}"
        
        self.logger.debug(f"Attempting to insert {len(seasons)} seasons into cache database...")
        
        batch_seasons_insert: list[tuple] = []
        
        for season_info in seasons:
            batch_seasons_insert.append((
                season_info.id,
                season_info.value,
                season_info.display_value,
                season_info.split,
                season_info.is_preseason
            ))
        
        self.cursor.executemany(
            """
            INSERT OR IGNORE INTO tblSeasonInfo (season_id, season_value, season_display_name, season_split, season_is_preseason)
            VALUES (:1, :2, :3, :4, :5)
            """,
            batch_seasons_insert
        )
        
        total_rc += self.cursor.rowcount
        self.logger.debug(return_msg.format(table="tblSeasonInfo", count=self.cursor.rowcount))
        
        self.conn.commit()
        self.conn.close()
        
        return_msg = f"You've made several changes to the database. Total rows affected: {total_rc}"
        
        if return_result:
            return return_msg
        else:
            self.logger.info(return_msg)
        
    
    def get_all_seasons(self) -> list[SeasonInfo] | None:
        """
        Gets all seasons from the cache database and returns a list of SeasonInfo objects.
        
        ### Returns:
            `list[SeasonInfo]` | `None` : Returns a list of SeasonInfo objects if found. Otherwise returns `None`.
        """
        self.conn = self.connect()
        self.cursor = self.conn.cursor()
        all_seasons = []
        
        self.logger.info("Getting all seasons from cache database...")
        self.cursor.execute("SELECT * FROM tblSeasonInfo;")
        result = self.cursor.fetchall()
        
        if result is None:
            self.logger.info("No seasons found in cache database.")
            return None
        else:
            self.logger.info(f"Found {len(result)} seasons in cache database.")
            for i, season in enumerate(result):
                season_obj = SeasonInfo(
                    id=season[0],
                    value=season[1],
                    display_value=season[2],
                    split=season[3],
                    is_preseason=season[4] == 1 # boolean values are saved as 0 (false) or 1 (true)
                )
                all_seasons.append(season_obj)
                self.logger.debug(f"Successfully rebuilt the \"{season_obj.display_value}\" season object from cache. ({i+1}/{len(result)})")
                
            return all_seasons
    
    
    def drop_tables(self, tables: list[str]) -> None:
        """
        Drops all specified tables.
        
        ### Args:
            tables : `str`
                A list of table names to be deleted/dropped
        """
        self.conn = self.connect()
        self.cursor = self.conn.cursor()
        
        for table in tables:
            self.logger.debug(f"Dropping table \"{table}\" ...")
            self.cursor.execute(f"DROP TABLE IF EXISTS {table}")
        
        self.conn.commit()
        self.conn.close()
        
    
    def connect(self) -> sqlite3.Connection:
        """
        Connects to local database, if it doesn't exist, one will be created.
        
        ### Returns:
            `sqlite3.Connection` : Returns a connection object.
        """
        return sqlite3.connect(self.db_path)



# Quick and simple scraper to pull some data from OPGG using multisearch

# Author  : ShoobyDoo
# Date    : 2023-07-05
# License : BSD-3-Clause


from datetime import datetime
from opgg.v1.params import By


class Passive:
    """
    Represents a champion's passive ability.\n
    
    ### Properties:
        `name: str` - Name of the passive\n
        `description: str` - Description of the passive\n
        `image_url: str` - URL to the passive image\n
        `video_url: str` - URL to the passive video\n
    """
    def __init__(self,
                 name: str,
                 description: str,
                 image_url: str,
                 video_url: str) -> None:
        self._name = name
        self._description = description 
        self._image_url = image_url
        self._video_url = video_url

    @property
    def name(self) -> str:
        """
        A `str` representing the name of the passive ability.
        """
        return self._name
    
    @property
    def description(self) -> str:
        """
        A `str` representing the description of the passive ability.
        """
        return self._description
    
    @property
    def image_url(self) -> str:
        """
        A `str` representing the URL to the passive image.
        """
        return self._image_url
    
    @property
    def video_url(self) -> str:
        """
        A `str` representing the URL to the passive video.
        """
        return self._video_url
    

class Spell:
    """
    Represents a champion's spell.\n
    
    ### Properties:
        `key: str` - Key of the spell\n
        `name: str` - Name of the spell\n
        `description: str` - Description of the spell\n
        `max_rank: int` - Max rank of the spell\n
        `range_burn: list` - Range of the spell\n
        `cooldown_burn: list` - Cooldowns of the spell\n
        `cooldown_burn_float: list[float]` - Cooldowns of the spell as floats\n
        `cost_burn: list` - Cost of the spell\n
        `tooltip: str` - Tooltip of the spell\n
        `image_url: str` - URL to the spell image\n
        `video_url: str` - URL to the spell video\n
    """
    def __init__(self,
                 key: str,
                 name: str,
                 description: str,
                 max_rank: int,
                 range_burn: list,
                 cooldown_burn: list,
                 cooldown_burn_float: list[float],
                 cost_burn: list,
                 tooltip: str,
                 image_url: str,
                 video_url: str) -> None:
        self._key = key
        self._name = name
        self._description = description
        self._max_rank = max_rank
        self._range_burn = range_burn
        self._cooldown_burn = cooldown_burn
        self._cooldown_burn_float = cooldown_burn_float
        self._cost_burn = cost_burn
        self._tooltip = tooltip
        self._image_url = image_url
        self._video_url = video_url

    @property
    def key(self) -> str:
        """
        A `str` representing the keybind of the spell. (Q, W, E, R)
        """
        return self._key
    
    @property
    def name(self) -> str:
        """
        A `str` representing the name of the spell.
        """
        return self._name
    
    @property
    def description(self) -> str:
        """
        A `str` representing the description of the spell.
        """
        return self._description
    
    @property
    def max_rank(self) -> int:
        """
        An `int` representing the max rank of the spell.
        """
        return self._max_rank
    
    @property
    def range_burn(self) -> list:
        """
        A `list[int]` representing the range(s) of the spell at each rank.
        """
        return self._range_burn
    
    @property
    def cooldown_burn(self) -> list:
        """
        A `list[int]` representing the cooldown(s) of the spell at each rank.
        """
        return self._cooldown_burn
    
    @property
    def cooldown_burn_float(self) -> list[float]:
        """
        A `list[float]` representing the cooldown(s) of the spell at each rank in floating point numbers. (More precise)
        """
        return self._cooldown_burn
    
    @property
    def cost_burn(self) -> list:
        """
        A `list[int]` representing the cost(s) of the spell at each rank.
        """
        return self._cost_burn
    
    @property
    def tooltip(self) -> str:
        """
        A `str` representing the tooltip of the spell.
        """
        return self._tooltip
    
    @property
    def image_url(self) -> str:
        """
        A `str` representing the URL to the spell's image.
        """
        return self._image_url
    
    @property
    def video_url(self) -> str:
        """
        A `str` representing the URL to a video of the spell in use.
        """
        return self._video_url
    
    def __repr__(self) -> str:
        return f"Skill({self.key}: {self.name})"


class Price:
    """
    Represents a price.\n
    
    ### Properties:
        `currency: str` - Currency of the price\n
        `cost: int` - Cost of the price\n
    """
    def __init__(self,
                 currency: str,
                 cost: int) -> None:
        self._currency = currency
        self._cost = cost
    
    @property
    def currency(self) -> str:
        """
        A `str` representing the currency of the price. (BE, RP)
        """
        return self._currency
    
    @property
    def cost(self) -> int:
        """
        An `int` representing the cost of the price.
        """
        return self._cost
    
    def __repr__(self) -> str:
        return f"Price({self.currency}: {self.cost})"


class Skin:
    """
    Represents a skin for a champion.\n
    
    ### Properties:
        `id: int` - ID of the skin\n
        `champion_id: int` - ID of the champion the skin belongs to\n
        `name: str` - Name of the skin\n
        `centered_image: str` - URL to the centered image of the skin\n
        `skin_video_url: str` - URL to the skin video\n
        `prices: list[Price]` - List of prices for the skin\n
        `sales: list` - List of sales for the skin. Defaults to None.\n
        `release_date: datetime` - Release date of the skin\n
    """
    def __init__(self,
                 id: int,
                 champion_id: int,
                 name: str,
                 centered_image: str,
                 skin_video_url: str, 
                 prices: list[Price],
                 release_date: datetime,
                 sales = None) -> None:
        self._id = id
        self._champion_id = champion_id
        self._name = name
        self._centered_image = centered_image 
        self._skin_video_url = skin_video_url 
        self._prices = prices 
        self._sales = sales
        self._release_date = release_date
    
    @property
    def id(self) -> int:
        """
        An `int` representing the ID of the skin.
        """
        return self._id
     
    @property
    def champion_id(self) -> int:
        """
        An `int` representing the ID of the champion the skin belongs to.
        """
        return self._champion_id
     
    @property
    def name(self) -> str:
        """
        A `str` representing the name of the skin.
        """
        return self._name
    
    @property
    def centered_image(self) -> str:
        """
        A `str` representing the URL to the centered image of the skin.
        """
        return self._centered_image
     
    @property
    def skin_video_url(self) -> str:
        """
        A `str` representing the URL to the skin video.
        """
        return self._skin_video_url
    
    @property
    def prices(self) -> list[Price]:
        """
        A `list[Price]` objects representing the prices of the skin.
        """
        return self._prices
    
    @property
    def sales(self) -> list:
        """
        A `list` representing the sales of the skin. Defaults to None.\n
        
        Note: This gets returned by the API, but all values I tested returned were null/none
        """
        return self._sales         
    
    @property
    def release_date(self) -> datetime:
        """
        A `datetime` object representing the release date of the skin
        """
        return self._release_date
    
    def __repr__(self) -> str:
        return f"Skin({self.name})"


class Champion:
    """
    Represents a champion.\n
    
    ### Properties:
        `id: int` - ID of the champion\n
        `key: str` - Key of the champion\n
        `name: str` - Name of the champion\n
        `image_url: str` - URL to the champion image\n
        `evolve: list` - List of evolutions for the champion\n
        `partype: str` - Resource used by champion to cast spells\n
        `passive: Passive` - Passive object for the champion\n
        `spells: list[Spell]` - List of Spell objects for the champion\n
        `skins: list[Skin]` - List of Skin objects for the champion\n
    """
    def __init__(self,
                 id: int,
                 key: str,
                 name: str,
                 image_url: str,
                 evolve: list,
                 partype: str,
                 passive: Passive,
                 spells: list[Spell],
                 skins: list[Skin]) -> None:
        self._id = id
        self._key = key
        self._name = name
        self._image_url = image_url
        self._evolve = evolve
        self._partype = partype
        self._passive = passive
        self._spells = spells
        self._skins = skins
        
    @property
    def id(self) -> int:
        """
        An `int` representing the ID of the champion.
        """
        return self._id
     
    @property
    def key(self) -> str:
        """
        A `str` representing the key of the champion.
        """
        return self._key
    
    @property
    def name(self) -> str:
        """
        A `str` representing the name of the champion.
        """
        return self._name
    
    @property
    def image_url(self) -> str:
        """
        A `str` representing the URL to the champion image.
        """
        return self._image_url
    
    @property
    def evolve(self) -> list:
        """
        A `list` representing the evolutions of the champion.
        
        Note: 99% of the time this is an empty list.
        """
        return self._evolve
    
    @property
    def partype(self) -> str:
        """
        A `str` representing the resource type the champion uses to cast spells. (Mana, Energy, etc.)
        """
        return self._partype
    
    @property
    def passive(self) -> Passive:
        """
        A `Passive` object representing the passive ability of the champion.
        """
        return self._passive
    
    @property
    def spells(self) -> list[Spell]:
        """
        A list[Spell] objects representing the spells of the champion.
        """
        return self._spells
    
    @property
    def skins(self) -> list[Skin]:
        """
        A `list[Skin]` objects representing the skins of the champion.
        """
        return self._skins


    def get_cost_by(self, by: By = By.BLUE_ESSENCE) -> int | None:
        """
        Get the cost of the champion.
        
        ### Args:
            by : `By`
                The currency to get the cost in. Defaults to `By.BLUE_ESSENCE`.
        """
        # Get the cost of the champion in either blue essence or riot points
        by = "IP" if by == By.BLUE_ESSENCE else by.upper()
        
        for skin in self.skins:
            if skin.prices is not None:
                for price in skin.prices:
                    if price.currency == by:
                        return price.cost
            else:
                return None
        
        
    def __repr__(self) -> str:
        return f"Champion(name={self.name})"


class ChampionStats:
    """
    Represents the stats of the user on a given champion.\n
    
    ### Properties:
        `champion: Champion` - Champion object\n
        `id: int` - Unique identifier for the stats\n
        `play: int` - Number of games played\n
        `win: int` - Number of games won\n
        `lose: int` - Number of games lost\n
        `kill: int` - Number of kills\n
        `death: int` - Number of deaths\n
        `assist: int` - Number of assists\n
        `gold_earned: int` - Amount of gold earned\n
        `minion_kill: int` - Number of minions killed\n
        `turret_kill: int` - Number of turrets killed\n
        `neutral_minion_kill: int` - Number of neutral minions killed\n
        `damage_dealt: int` - Amount of damage dealt\n
        `damage_taken: int` - Amount of damage taken\n
        `physical_damage_dealt: int` - Amount of physical damage dealt\n
        `magic_damage_dealt: int` - Amount of magic damage dealt\n
        `most_kill: int` - Most kills in a game\n
        `max_kill: int` - Max kills in a game\n
        `max_death: int` - Max deaths in a game\n
        `double_kill: int` - Number of double kills\n
        `triple_kill: int` - Number of triple kills\n
        `quadra_kill: int` - Number of quadra kills\n
        `penta_kill: int` - Number of penta kills\n
        `game_length_second: int` - Total game length in seconds\n
        `inhibitor_kills: int` - Number of inhibitor kills\n
        `sight_wards_bought_in_game: int` - Number of sight wards bought in game\n
        `vision_wards_bought_in_game: int` - Number of vision wards bought in game\n
        `vision_score: int` - Vision score\n
        `wards_placed: int` - Number of wards placed\n
        `wards_killed: int` - Number of wards killed\n
        `heal: int` - Amount of healing done\n
        `time_ccing_others: int` - Time spent crowd-controlling others\n
        `op_score: int` - Overall performance score\n
        `is_max_in_team_op_score: int` - Indicator if the max score in team is OP\n
        `physical_damage_taken: int` - Amount of physical damage taken\n
        `damage_dealt_to_champions: int` - Total damage dealt to champions\n
        `physical_damage_dealt_to_champions: int` - Physical damage dealt to champions\n
        `magic_damage_dealt_to_champions: int` - Magic damage dealt to champions\n
        `damage_dealt_to_objectives: int` - Damage dealt to objectives\n
        `damage_dealt_to_turrets: int` - Damage dealt to turrets\n
        `damage_self_mitigated: int` - Amount of damage self-mitigated\n
        `max_largest_multi_kill: int` - Maximum largest multi-kill\n
        `max_largest_critical_strike: int` - Maximum largest critical strike\n
        `max_largest_killing_spree: int` - Maximum largest killing spree\n
        `snowball_throws: int` - Number of snowball throws\n
        `snowball_hits: int` - Number of snowball hits\n
    """
    def __init__(self,
                 champion: Champion,
                 id: int,
                 play: int,
                 win: int,
                 lose: int,
                 kill: int,
                 death: int,
                 assist: int,
                 gold_earned: int,
                 minion_kill: int,
                 turret_kill: int,
                 neutral_minion_kill: int,
                 damage_dealt: int,
                 damage_taken: int,
                 physical_damage_dealt: int,
                 magic_damage_dealt: int,
                 most_kill: int,
                 max_kill: int,
                 max_death: int,
                 double_kill: int,
                 triple_kill: int,
                 quadra_kill: int,
                 penta_kill: int,
                 game_length_second: int,
                 inhibitor_kills: int,
                 sight_wards_bought_in_game: int,
                 vision_wards_bought_in_game: int,
                 vision_score: int,
                 wards_placed: int,
                 wards_killed: int,
                 heal: int,
                 time_ccing_others: int,
                 op_score: int,
                 is_max_in_team_op_score: int,
                 physical_damage_taken: int,
                 damage_dealt_to_champions: int,
                 physical_damage_dealt_to_champions: int,
                 magic_damage_dealt_to_champions: int,
                 damage_dealt_to_objectives: int,
                 damage_dealt_to_turrets: int,
                 damage_self_mitigated: int,
                 max_largest_multi_kill: int,
                 max_largest_critical_strike: int,
                 max_largest_killing_spree: int,
                 snowball_throws: int,
                 snowball_hits: int) -> None:
        self._champion = champion
        self._id = id
        self._play = play
        self._win = win
        self._lose = lose
        self._kill = kill
        self._death = death
        self._assist = assist
        self._gold_earned = gold_earned
        self._minion_kill = minion_kill 
        self._turret_kill = turret_kill 
        self._neutral_minion_kill = neutral_minion_kill 
        self._damage_dealt = damage_dealt 
        self._damage_taken = damage_taken 
        self._physical_damage_dealt = physical_damage_dealt 
        self._magic_damage_dealt = magic_damage_dealt 
        self._most_kill = most_kill 
        self._max_kill = max_kill 
        self._max_death = max_death 
        self._double_kill = double_kill 
        self._triple_kill = triple_kill 
        self._quadra_kill = quadra_kill 
        self._penta_kill = penta_kill 
        self._game_length_second = game_length_second
        
        self._inhibitor_kills = inhibitor_kills
        self._sight_wards_bought_in_game = sight_wards_bought_in_game
        self._vision_wards_bought_in_game = vision_wards_bought_in_game
        self._vision_score = vision_score
        self._wards_placed = wards_placed
        self._wards_killed = wards_killed
        self._heal = heal
        self._time_ccing_others = time_ccing_others
        self._op_score = op_score
        self._is_max_in_team_op_score = is_max_in_team_op_score
        self._physical_damage_taken = physical_damage_taken
        self._damage_dealt_to_champions = damage_dealt_to_champions
        self._physical_damage_dealt_to_champions = physical_damage_dealt_to_champions
        self._magic_damage_dealt_to_champions = magic_damage_dealt_to_champions
        self._damage_dealt_to_objectives = damage_dealt_to_objectives
        self._damage_dealt_to_turrets = damage_dealt_to_turrets
        self._damage_self_mitigated = damage_self_mitigated
        self._max_largest_multi_kill = max_largest_multi_kill
        self._max_largest_critical_strike = max_largest_critical_strike
        self._max_largest_killing_spree = max_largest_killing_spree
        self._snowball_throws = snowball_throws
        self._snowball_hits = snowball_hits        
        
    @property
    def champion(self) -> Champion:
        """
        A `Champion` object representing the champion.
        """
        return self._champion
    
    @property
    def id(self) -> int:
        """
        An `int` representing the champion id
        """
        return self._id
    
    @property
    def play(self) -> int:
        """
        An `int` representing the number of games played.
        """
        return self._play

    @property
    def win(self) -> int:
        """
        An `int` representing the number of games won.
        """
        return self._win

    @property
    def lose(self) -> int:
        """
        An `int` representing the number of games lost.
        """
        return self._lose

    @property
    def kill(self) -> int:
        """
        An `int` representing the number of kills.
        """
        return self._kill

    @property
    def death(self) -> int:
        """
        An `int` representing the number of deaths.
        """
        return self._death

    @property
    def assist(self) -> int:
        """
        An `int` representing the number of assists.
        """
        return self._assist

    @property
    def gold_earned(self) -> int:
        """
        An `int` representing the amount of gold earned.
        """
        return self._gold_earned

    @property
    def minion_kill(self) -> int:
        """
        An `int` representing the number of minions killed.
        """
        return self._minion_kill
 
    @property
    def turret_kill(self) -> int:
        """
        An `int` representing the number of turrets killed.
        """
        return self._turret_kill

    @property
    def neutral_minion_kill(self) -> int:
        """
        An `int` representing the number of neutral minions killed.
        """
        return self._neutral_minion_kill

    @property
    def damage_dealt(self) -> int:
        """
        An `int` representing the amount of damage dealt.
        """
        return self._damage_dealt
        
    @property
    def damage_taken(self) -> int:
        """
        An `int` representing the amount of damage taken.
        """
        return self._damage_taken
        
    @property
    def physical_damage_dealt(self) -> int:
        """
        An `int` representing the amount of physical damage dealt.
        """
        return self._physical_damage_dealt
        
    @property
    def magic_damage_dealt(self) -> int:
        """
        An `int` representing the amount of magic damage dealt.
        """
        return self._magic_damage_dealt

    @property
    def most_kill(self) -> int:
        """
        An `int` representing the most kills in a game.
        """
        return self._most_kill
    
    @property
    def max_kill(self) -> int:
        """
        An `int` representing the max kills in a game.
        """
        return self._max_kill
    
    @property
    def max_death(self) -> int:
        """
        An `int` representing the max deaths in a game.
        """
        return self._max_death
    
    @property
    def double_kill(self) -> int:
        """
        An `int` representing the number of double kills.
        """
        return self._double_kill
    
    @property
    def triple_kill(self) -> int:
        """
        An `int` representing the number of triple kills.
        """
        return self._triple_kill
    
    @property
    def quadra_kill(self) -> int:
        """
        An `int` representing the number of quadra kills.
        """
        return self._quadra_kill
    
    @property
    def penta_kill(self) -> int:
        """
        An `int` representing the number of penta kills.
        """
        return self._penta_kill
    
    @property
    def game_length_second(self) -> int:
        """
        An `int` representing the total game length in seconds.
        """
        return self._game_length_second
    
    @property
    def kda(self) -> float:
        """
        A `float` representing the KDA of the champion.
        """
        return (self._kill + self._assist) / self._death if self._death != 0 else 0

    @property
    def win_rate(self) -> float:
        """
        A `float` representing the win rate of the champion.
        """
        return round(float((self._win / self._play) * 100), 2) if self._play != 0 else 0

    @property
    def inhibitor_kills(self) -> int:
        """
        An `int` representing the number of inhibitor kills.
        """
        return self._inhibitor_kills

    @property
    def sight_wards_bought_in_game(self) -> int:
        """
        An `int` representing the number of sight wards bought in game.
        """
        return self._sight_wards_bought_in_game

    @property
    def vision_wards_bought_in_game(self) -> int:
        """
        An `int` representing the number of vision wards bought in game.
        """
        return self._vision_wards_bought_in_game

    @property
    def vision_score(self) -> int:
        """
        An `int` representing the vision score.
        """
        return self._vision_score

    @property
    def wards_placed(self) -> int:
        """
        An `int` representing the number of wards placed.
        """
        return self._wards_placed

    @property
    def wards_killed(self) -> int:
        """
        An `int` representing the number of wards killed.
        """
        return self._wards_killed

    @property
    def heal(self) -> int:
        """
        An `int` representing the amount of healing done.
        """
        return self._heal

    @property
    def time_ccing_others(self) -> int:
        """
        An `int` representing the time spent crowd-controlling others.
        """
        return self._time_ccing_others

    @property
    def op_score(self) -> int:
        """
        An `int` representing the OP score.
        """
        return self._op_score

    @property
    def is_max_in_team_op_score(self) -> bool:
        """
        A `bool` indicating if the player has the maximum OP score in the team.
        """
        return self._is_max_in_team_op_score

    @property
    def physical_damage_taken(self) -> int:
        """
        An `int` representing the physical damage taken.
        """
        return self._physical_damage_taken

    @property
    def damage_dealt_to_champions(self) -> int:
        """
        An `int` representing the total damage dealt to champions.
        """
        return self._damage_dealt_to_champions

    @property
    def physical_damage_dealt_to_champions(self) -> int:
        """
        An `int` representing the physical damage dealt to champions.
        """
        return self._physical_damage_dealt_to_champions

    @property
    def magic_damage_dealt_to_champions(self) -> int:
        """
        An `int` representing the magic damage dealt to champions.
        """
        return self._magic_damage_dealt_to_champions

    @property
    def damage_dealt_to_objectives(self) -> int:
        """
        An `int` representing the damage dealt to objectives.
        """
        return self._damage_dealt_to_objectives

    @property
    def damage_dealt_to_turrets(self) -> int:
        """
        An `int` representing the damage dealt to turrets.
        """
        return self._damage_dealt_to_turrets

    @property
    def damage_self_mitigated(self) -> int:
        """
        An `int` representing the damage self mitigated.
        """
        return self._damage_self_mitigated

    @property
    def max_largest_multi_kill(self) -> int:
        """
        An `int` representing the maximum largest multi-kill.
        """
        return self._max_largest_multi_kill

    @property
    def max_largest_critical_strike(self) -> int:
        """
        An `int` representing the maximum largest critical strike.
        """
        return self._max_largest_critical_strike

    @property
    def max_largest_killing_spree(self) -> int:
        """
        An `int` representing the maximum largest killing spree.
        """
        return self._max_largest_killing_spree

    @property
    def snowball_throws(self) -> int:
        """
        An `int` representing the number of snowball throws.
        """
        return self._snowball_throws

    @property
    def snowball_hits(self) -> int:
        """
        An `int` representing the number of snowball hits.
        """
        return self._snowball_hits


    def __repr__(self) -> str:
        return  f"ChampionStats(champion={self.champion}, win={self.win} / lose={self.lose} (winrate: {self.win_rate}%), kda={round(self.kda, 2)})"


from datetime import datetime
from typing import Optional
from pydantic import BaseModel, HttpUrl


class QueueInfo(BaseModel):
    id: int
    queue_translate: str
    game_type: str


class TierInfo(BaseModel):
    tier: Optional[str] = None
    division: Optional[int] = None
    lp: Optional[int] = None
    level: Optional[int] = None
    tier_image_url: HttpUrl
    border_image_url: Optional[HttpUrl] = None


class League(BaseModel):
    game_type: str
    tier_info: TierInfo
    win: Optional[int] = None
    lose: Optional[int] = None
    is_hot_streak: bool = False
    is_fresh_blood: bool = False
    is_veteran: bool = False
    is_inactive: bool = False
    series: Optional[dict] = None
    updated_at: Optional[datetime] = None
    
    class Config:
        json_encoders = {datetime: lambda v: v.isoformat()}


class RankInfo(BaseModel):
    """Basic rank information."""

    tier: str
    division: int
    lp: int


class RankEntry(BaseModel):
    """Entry containing rank information for a specific game type."""

    game_type: str
    rank_info: RankInfo
    created_at: datetime

    class Config:
        json_encoders = {datetime: lambda v: v.isoformat()}


class Season(BaseModel):
    """Represents a League season."""

    season_id: int
    tier_info: TierInfo
    # Make created_at optional since API doesn't always include it
    created_at: Optional[datetime] = None

    class Config:
        """Pydantic model configuration."""

        arbitrary_types_allowed = True
        json_encoders = {datetime: lambda v: v.isoformat()}



import json
import logging
import asyncio
import aiohttp

from typing import Optional, Tuple, Any
from pprint import pformat

from opgg.v2.champion import Champion
from opgg.v2.params import LangCode, Region, GenericReqParams
from opgg.v2.search_result import SearchResult
from opgg.v2.summoner import Summoner


logger = logging.getLogger("OPGG.py")


class Utils:
    """
    ### utils.py
    A collection of static utility helper methods that perform various opgg and league specific tasks such as fetching champions, seasons, etc.

    Copyright (c) 2023-2024, ShoobyDoo
    License: BSD-3-Clause, See LICENSE for more details.
    """

    @staticmethod
    def read_local_json(file: str) -> dict[str, Any]:
        """
        Read and parse a local JSON file.

        Args:
            `file` (`str`): Path to the JSON file

        Returns:
            `dict[str, Any]`: Parsed JSON data as a dictionary
        """
        with open(file, "r") as f:
            return json.loads(f.read())

    @staticmethod
    def safe_get(obj, *attrs):
        for attr in attrs:
            try:
                obj = getattr(obj, attr)
            except AttributeError:
                return None
        return obj

    @staticmethod
    async def _search_region(
        session: aiohttp.ClientSession,
        query: str,
        region: Region,
        params: GenericReqParams,
    ) -> Tuple[Region, list[dict]]:
        """
        `[Internal Helper Method]` Search for a summoner in a specific region using the OP.GG API.

        Args:
            `session` (`aiohttp.ClientSession`): Active session for making requests
            `query` (`str`): Summoner name to search for
            `region` (`Region`): Region to search in
            `search_api_url` (`str`): Full search API URL template
            `headers` (`dict`): Headers to use for the request

        Returns:
            `Tuple[Region, list[dict]]`: Region and list of found summoners
        """
        data = {
            "summoner_name": query,
            "region": str(region),
            "tagline": "",
        }

        if "#" in query:
            summoner_name, tagline = query.split("#")
            data["summoner_name"] = summoner_name
            data["tagline"] = tagline

        logger.debug(f"Constructed search data: {data}")
        search_url = params["base_api_url"].format_map(data)
        logger.info(
            f"Sending search request to OPGG API... (API_URL = {search_url}, HEADERS = {params['headers']})"
        )

        try:
            async with session.get(search_url, headers=params["headers"]) as res:
                logger.debug(f"Response status: {res.status}")
                logger.debug(f"Response headers: {dict(res.headers)}")

                if res.status == 200:
                    content: dict = await res.json()
                    logger.debug(f"Raw response payload: {pformat(content)}")
                    data = content.get("data", [])
                    logger.info(f"Found {len(data)} results for region {region}")
                res.raise_for_status()
        except Exception as e:
            logger.error(f"Error searching region {region}: {str(e)}")
            data = []

        return region, data

    @staticmethod
    async def _search_all_regions(query: str, params: GenericReqParams) -> list[dict]:
        """
        `[Internal Helper Method]` Concurrently searches for a summoner across all available regions.

        Args:
            `query` (`str`): Summoner name to search for
            `search_api_url` (`str`): Full search API URL template
            `headers` (`dict`): Headers to use for the request

        Returns:
            `list[dict]`: list of unique summoner results across all regions
        """
        logger.info(f"Starting concurrent search across all regions for query: {query}")

        async with aiohttp.ClientSession() as session:
            tasks = [
                Utils._search_region(
                    session,
                    query,
                    region,
                    params,
                )
                for region in Region
                if region != Region.ANY
            ]
            logger.debug(f"Created {len(tasks)} search tasks")

            results = await asyncio.gather(*tasks)
            logger.debug(f"Raw results from all regions: {pformat(results)}")

            # Flatten results and remove duplicates
            all_results = []
            seen = set()
            for region, data in results:
                logger.debug(f"Processing {len(data)} results from region {region}")

                result: dict
                for result in data:
                    unique_id = result.get("summoner_id")
                    if unique_id not in seen:
                        seen.add(unique_id)
                        all_results.append({"region": str(region), "summoner": result})

            logger.info(f"Found {len(all_results)} unique results across all regions")
            return all_results

    @staticmethod
    async def _single_region_search(
        query: str, region: Region, params: GenericReqParams
    ):
        async with aiohttp.ClientSession() as session:
            region, data = await Utils._search_region(
                session,
                query,
                region,
                params,
            )
            # Format results the same way as _search_all_regions
            return [{"region": str(region), "summoner": result} for result in data]

    @staticmethod
    async def _fetch_profile(summoner_id: str, params: GenericReqParams):
        logger.info(
            f"Fetching profile for summoner ID: {summoner_id} (API_URL = {params['base_api_url']}, HEADERS = {params['headers']})"
        )

        async with aiohttp.ClientSession() as session:
            async with session.get(
                params.get("base_api_url"), headers=params.get("headers")
            ) as res:
                logger.debug(f"Response status: {res.status}")

                if res.status == 200:
                    content: dict = await res.json()
                    logger.info(
                        f"Request to OPGG API was successful (Content Length: {len(str(content))})"
                    )
                    logger.debug(
                        f"SUMMONER DATA AT /SUMMARY ENDPOINT:\n{pformat(content)}\n"
                    )
                    return content.get("data", {})

                res.raise_for_status()

    @staticmethod
    async def _fetch_profile_multiple(
        params: GenericReqParams,
        search_results: list[SearchResult] = None,
        summoner_ids: Optional[list[str]] = None,
        regions: list[Region] = None,
    ):
        if search_results is None and summoner_ids is not None and regions is not None:
            search_results = [
                SearchResult(
                    {
                        "summoner": Summoner({"summoner_id": summoner_id}),
                        "region": region,
                    }
                )
                for summoner_id, region in zip(summoner_ids, regions)
            ]

        tasks = [
            Utils._fetch_profile(
                search_result.summoner.summoner_id,
                params={
                    "base_api_url": params["base_api_url"].format_map(
                        {
                            "summoner_id": search_result.summoner.summoner_id,
                            "region": search_result.region,
                        }
                    ),
                    "headers": params["headers"],
                },
            )
            for search_result in search_results
        ]

        return await asyncio.gather(*tasks)

    @staticmethod
    async def _fetch_recent_games(
        params: GenericReqParams,
    ):
        async with aiohttp.ClientSession() as session:
            async with session.get(
                params["base_api_url"], headers=params["headers"]
            ) as res:
                logger.debug(f"Response status: {res.status}")

                if res.status == 200:
                    content: dict = await res.json()
                    logger.info(
                        f"Request to OPGG API was successful (Content Length: {len(str(content))})"
                    )
                    logger.debug(f"GAME DATA AT /GAMES ENDPOINT:\n{pformat(content)}\n")
                    return content.get("data", {})

                res.raise_for_status()

    @staticmethod
    async def _fetch_recent_games_multiple(
        params_list: list[GenericReqParams],
    ):
        tasks = [Utils._fetch_recent_games(params) for params in params_list]

        return await asyncio.gather(*tasks)

    @staticmethod
    async def _update(search_result: SearchResult, params: GenericReqParams):
        update_api_url = params["base_api_url"].format_map(
            {
                "region": search_result.region,
                "summoner_id": search_result.summoner.summoner_id,
            }
        )

        logger.info(
            f"Updating profile for summoner ID: {search_result.summoner.summoner_id}"
        )
        logger.debug(f"API URL: {update_api_url}")

        async with aiohttp.ClientSession() as session:
            async with session.post(
                update_api_url,
                headers=params["headers"],
            ) as res:
                logger.debug(f"Response status: {res.status}")

                if res.status in [200, 201, 202]:
                    return await res.json()
                else:
                    res.raise_for_status()

    @staticmethod
    async def _fetch_all_champions(params: GenericReqParams):
        async with aiohttp.ClientSession() as session:

            logger.debug(f"API URL: {params['base_api_url']}")

            async with session.get(
                params["base_api_url"], headers=params["headers"]
            ) as res:
                logger.debug(f"Response status: {res.status}")

                if res.status == 200:
                    content: dict = await res.json()
                    logger.info(
                        f"Request to OPGG API was successful (Content Length: {len(str(content))})"
                    )
                    logger.debug(
                        f"CHAMPION DATA AT /CHAMPIONS ENDPOINT:\n{pformat(content, sort_dicts=False)}\n"
                    )
                    return content.get("data", {})

                res.raise_for_status()

    @staticmethod
    async def _fetch_champion_by_id(
        id: int, params: GenericReqParams, lang_code: LangCode
    ):
        async with aiohttp.ClientSession() as session:
            fetch_champ_by_id_url = params["base_api_url"].format_map(
                {"champion_id": id, "hl": lang_code}
            )

            logger.debug(f"API URL: {fetch_champ_by_id_url}")

            async with session.get(
                fetch_champ_by_id_url,
                headers=params["headers"],
            ) as res:
                logger.debug(f"Response status: {res.status}")

                if res.status == 200:
                    content: dict = await res.json()
                    logger.info(
                        f"Request to OPGG API was successful (Content Length: {len(str(content))})"
                    )
                    logger.debug(
                        f"CHAMPION DATA AT /CHAMPIONS ENDPOINT:\n{pformat(content)}\n"
                    )
                    return content.get("data", {})

                res.raise_for_status()

    @staticmethod
    async def _fetch_live_game(params: GenericReqParams):
        async with aiohttp.ClientSession() as session:
            async with session.get(
                params["base_api_url"], headers=params["headers"]
            ) as res:
                logger.debug(f"Response status: {res.status}")

                if res.status == 200:
                    content: dict = await res.json()
                    logger.info(
                        f"Request to OPGG API was successful (Content Length: {len(str(content))})"
                    )
                    logger.debug(
                        f"LIVE GAME DATA AT /spectate ENDPOINT:\n{pformat(content)}\n"
                    )

                elif res.status == 404:
                    content: dict = await res.json()

                    detail = content.get("detail", {}).get("detailMessage", "")
                    code = content.get("code", 404)
                    message = content.get("message", "Not found")

                    content = {
                        "status": code,
                        "detail": (
                            "Provided summoner is not in a live game!"
                            if detail == ""
                            else detail
                        ),
                        "message": message,
                    }

                else:
                    res.raise_for_status()

                # Success response example:
                # {
                #   "status": 200,
                #   "data": {...},
                #   "message": "Success"
                # }

                # Error response example:
                # {
                #   "status": 404,
                #   "message": "Not found",
                #   "detail": "Provided summoner is not in a live game!"
                # }

                content = {
                    "status": res.status,
                    "message": content.get("message", "Success"),
                    **({"data": content.get("data")} if res.status == 200 else {}),
                    **({"detail": content.get("detail")} if res.status != 200 else {}),
                }

                return content



# Quick and simple scraper to pull some data from OPGG using multisearch

# Author  : ShoobyDoo
# Date    : 2023-07-05
# License : BSD-3-Clause

import os
import json
import logging
import requests
import traceback

from datetime import datetime
from typing import Literal
from fake_useragent import UserAgent

# from opgg.summoner import
from opgg.v1.game import GameStats, Stats, Team
from opgg.v1.season import RankEntry, Season, SeasonInfo
from opgg.v1.champion import ChampionStats, Champion
from opgg.v1.league_stats import LeagueStats, Tier, QueueInfo
from opgg.v1.summoner import Game, Participant, Summoner
from opgg.v1.params import Region
from opgg.v1.cacher import Cacher
from opgg.v1.utils import Utils

from warnings import warn


class OPGG:
    """
    ### OPGG.py (v1)
    A simple library to access and structure the data from OP.GG's Website & API.

    Copyright (c) 2023-2024, ShoobyDoo
    License: BSD-3-Clause, See LICENSE for more details.

    ---
    #### Deprecation Notice

    This version of OPGG.py is no longer maintained and will be deprecated in the future. Please migrate to v2 as soon as possible.
    """

    __author__ = "ShoobyDoo"
    __license__ = "BSD-3-Clause"

    # Todo: Add support for the following endpoint(s):
    # https://op.gg/api/v1.0/internal/bypass/games/na/summoners/<summoner_id?>/?&limit=20&hl=en_US&game_type=total

    # HUGE find. Incorporate the following endpoint:
    # https://op.gg/api/v1.0/internal/bypass/meta/champions/<champion_id>?hl=<lang_code>

    # METADATA FOR CHAMPIONS -- USE THIS OVER PAGE_PROPS.
    # https://op.gg/api/v1.0/internal/bypass/meta/champions?hl=en_US

    def __init__(self, summoner_id: str = None, region=Region.NA) -> None:
        warn(
            "Deprecation Notice: The OPGG class in v1 is no longer maintained. "
            "Please migrate to v2 as soon as possible.",
            DeprecationWarning,
            stacklevel=2,
        )

        self._summoner_id = summoner_id
        self._region = region

        self._BASE_API_URL = "https://lol-web-api.op.gg/api/v1.0/internal/bypass"
        self._api_url = (
            f"{self._BASE_API_URL}/summoners/{self.region}/{self.summoner_id}/summary"
        )
        self._games_api_url = (
            f"{self._BASE_API_URL}/games/{self.region}/summoners/{self.summoner_id}"
        )

        self._ua = UserAgent()

        # Should randomize EACH request instead of the header of initially init'd obj
        self._headers = {"User-Agent": self._ua.random}

        # in memory data
        self._all_champions = None
        self._all_seasons = None

        # ===== SETUP START =====
        logging.root.name = "OPGG.py"

        if not os.path.exists("./logs"):
            logging.info("Creating logs directory...")
            os.mkdir("./logs")
        else:
            # remove empty log files
            for file in os.listdir("./logs"):
                if (
                    os.stat(f"./logs/{file}").st_size == 0
                    and file != f'opgg_{datetime.now().strftime("%Y-%m-%d")}.log'
                ):
                    logging.info(f"Removing empty log file: {file}")
                    os.remove(f"./logs/{file}")

        logging.basicConfig(
            filename=f'./logs/opgg_{datetime.now().strftime("%Y-%m-%d")}.log',
            filemode="a+",
            format="[%(asctime)s][%(name)s->%(module)s:%(lineno)-10d][%(levelname)-7s] : %(message)s",
            datefmt="%d-%b-%y %H:%M:%S",
            level=logging.INFO,
        )
        # ===== SETUP END =====

        # allow the user to interact with the logger
        self._logger = logging.getLogger("OPGG.py")

        # at object creation, setup and query the cache
        self._cacher = Cacher()
        self._cacher.setup()

        self.logger.info(
            f"OPGG.__init__(summoner_id={self.summoner_id}, "
            f"region={self.region}, "
            f"api_url={self.api_url}, "
            f"headers={self.headers}, "
            f"all_champions={self.all_champions}, "
            f"all_seasons={self.all_seasons})"
        )

    @property
    def logger(self) -> logging.Logger:
        """
        A `Logger` object representing the logger instance.

        The logging level is set to `INFO` by default.
        """
        return self._logger

    @property
    def summoner_id(self) -> str:
        """
        A `str` representing the summoner id. (Riot API)
        """
        return self._summoner_id

    @summoner_id.setter
    def summoner_id(self, value: str) -> None:
        self._summoner_id = value
        self.refresh_api_url()

    @property
    def region(self) -> str:
        """
        A `str` representing the region to search in.
        """
        return self._region

    @region.setter
    def region(self, value: str) -> None:
        self._region = value
        self.refresh_api_url()

    @property
    def api_url(self) -> str:
        """
        A `str` representing the api url to send requests to. (OPGG API)
        """
        return self._api_url

    @api_url.setter
    def api_url(self, value: str) -> None:
        self._api_url = value

    @property
    def headers(self) -> dict:
        """
        A `dict` representing the headers to send with the request.
        """
        return self._headers

    @headers.setter
    def headers(self, value: dict) -> None:
        self._headers = value

    @property
    def all_champions(self) -> list[Champion]:
        """
        A `list[Champion]` objects representing all champions in the game.
        """
        return self._all_champions

    @all_champions.setter
    def all_champions(self, value: list[Champion]) -> None:
        self._all_champions = value

    @property
    def all_seasons(self) -> list[SeasonInfo]:
        """
        A `list[SeasonInfo]` objects representing all seasons in the game.
        """
        return self._all_seasons

    @all_seasons.setter
    def all_seasons(self, value: list[SeasonInfo]) -> None:
        self._all_seasons = value

    @property
    def cacher(self) -> Cacher:
        """
        A `Cacher` object representing the summoner_id cacher.
        """
        return self._cacher

    def refresh_api_url(self) -> None:
        """
        A method to refresh the api url with the current summoner id and region.
        """
        self.api_url = (
            f"{self._BASE_API_URL}/summoners/{self.region}/{self.summoner_id}/summary"
        )
        self._games_api_url = (
            f"{self._BASE_API_URL}/games/{self.region}/summoners/{self.summoner_id}"
        )

        self.logger.debug(f"self.refresh_api_url() called... See URLs:")
        self.logger.debug(f"self.api_url = {self.api_url}")
        self.logger.debug(f"self._games_api_url = {self._games_api_url}")

    def get_summoner(self, return_content_only=False) -> Summoner | dict:
        """
        A method to get data from the OPGG API and form a Summoner object.

        General flow:\n
            -> Send request to OPGG API\n
            -> Parse data from request (jsonify)\n
            -> Loop through data and form the summoner object.

        ### Returns:
            `Summoner`: A Summoner object representing the summoner.
        """
        self.logger.info(
            f"Sending request to OPGG API... (API_URL = {self.api_url}, HEADERS = {self.headers})"
        )
        res = requests.get(self.api_url, headers=self.headers)

        previous_seasons: list[Season] = []
        league_stats: list[LeagueStats] = []
        most_champions: list[ChampionStats] = []
        recent_game_stats: list[Game] = []

        if res.status_code == 200:
            self.logger.info(
                f"Request to OPGG API was successful, parsing data (Content Length: {len(res.text)})..."
            )
            self.logger.debug(f"SUMMONER DATA AT /SUMMARY ENDPOINT:\n{res.text}\n")
            content = json.loads(res.text)["data"]
        else:
            res.raise_for_status()

        # If return_res is passed in func args, return the content
        # Required in tests to get all the raw content without building the summoner object
        if return_content_only:
            return content

        try:
            previous_seasons = []
            summoner_data = dict(content["summoner"])

            season: dict
            tier_info: dict
            for season in summoner_data.get("previous_seasons", []):
                tmp_season_info = None
                if self.all_seasons:
                    for _season in self.all_seasons:
                        if _season.id == season.get("season_id"):
                            tmp_season_info = _season
                            break

                tmp_rank_entries = []

                rank_entry: dict
                rank_info: dict
                for rank_entry in season.get("rank_entries", []):
                    rank_info = rank_entry.get("rank_info")
                    if not rank_info:
                        continue
                    tmp_rank_entries.append(
                        RankEntry(
                            game_type=rank_entry.get("game_type"),
                            rank_info=Tier(
                                tier=rank_info.get("tier"),
                                division=rank_info.get("division"),
                                lp=rank_info.get("lp"),
                            ),
                            created_at=(
                                datetime.fromisoformat(rank_entry.get("created_at"))
                                if rank_entry.get("created_at")
                                else None
                            ),
                        )
                    )

                tier_info = season.get("tier_info", {})
                previous_seasons.append(
                    Season(
                        season_id=tmp_season_info,
                        tier_info=Tier(
                            tier=tier_info.get("tier"),
                            division=tier_info.get("division"),
                            lp=tier_info.get("lp"),
                            tier_image_url=tier_info.get("tier_image_url"),
                            border_image_url=tier_info.get("border_image_url"),
                        ),
                        rank_entries=tmp_rank_entries,
                        created_at=(
                            datetime.fromisoformat(season.get("created_at"))
                            if season.get("created_at")
                            else None
                        ),
                    )
                )

            league_stats = []
            league: dict
            queue_info: dict
            for league in summoner_data.get("league_stats", []):
                queue_info = league.get("queue_info", {})
                tier_info = league.get("tier_info", {})
                league_stats.append(
                    LeagueStats(
                        queue_info=QueueInfo(
                            id=queue_info.get("id"),
                            queue_translate=queue_info.get("queue_translate"),
                            game_type=queue_info.get("game_type"),
                        ),
                        tier_info=Tier(
                            tier=tier_info.get("tier"),
                            division=tier_info.get("division"),
                            lp=tier_info.get("lp"),
                            tier_image_url=tier_info.get("tier_image_url"),
                            border_image_url=tier_info.get("border_image_url"),
                            level=tier_info.get("level"),
                        ),
                        win=league.get("win"),
                        lose=league.get("lose"),
                        is_hot_streak=league.get("is_hot_streak"),
                        is_fresh_blood=league.get("is_fresh_blood"),
                        is_veteran=league.get("is_veteran"),
                        is_inactive=league.get("is_inactive"),
                        series=league.get("series"),
                        updated_at=league.get("updated_at"),
                    )
                )

            most_champions = []
            # Assign 'most_champions' to a variable to help the linter
            most_champions_data: dict = summoner_data.get("most_champions", {})
            champion_stats: dict = most_champions_data.get("champion_stats", [])

            champion: dict
            for champion in champion_stats:
                tmp_champ = None
                if self.all_champions:
                    for _champion in self.all_champions:
                        if _champion.id == champion.get("id"):
                            tmp_champ = _champion
                            break

                most_champions.append(
                    ChampionStats(
                        champion=tmp_champ,
                        id=champion.get("id"),
                        play=champion.get("play"),
                        win=champion.get("win"),
                        lose=champion.get("lose"),
                        kill=champion.get("kill"),
                        death=champion.get("death"),
                        assist=champion.get("assist"),
                        gold_earned=champion.get("gold_earned"),
                        minion_kill=champion.get("minion_kill"),
                        turret_kill=champion.get("turret_kill"),
                        neutral_minion_kill=champion.get("neutral_minion_kill"),
                        damage_dealt=champion.get("damage_dealt"),
                        damage_taken=champion.get("damage_taken"),
                        physical_damage_dealt=champion.get("physical_damage_dealt"),
                        magic_damage_dealt=champion.get("magic_damage_dealt"),
                        most_kill=champion.get("most_kill"),
                        max_kill=champion.get("max_kill"),
                        max_death=champion.get("max_death"),
                        double_kill=champion.get("double_kill"),
                        triple_kill=champion.get("triple_kill"),
                        quadra_kill=champion.get("quadra_kill"),
                        penta_kill=champion.get("penta_kill"),
                        game_length_second=champion.get("game_length_second"),
                        inhibitor_kills=champion.get("inhibitor_kills"),
                        sight_wards_bought_in_game=champion.get(
                            "sight_wards_bought_in_game"
                        ),
                        vision_wards_bought_in_game=champion.get(
                            "vision_wards_bought_in_game"
                        ),
                        vision_score=champion.get("vision_score"),
                        wards_placed=champion.get("wards_placed"),
                        wards_killed=champion.get("wards_killed"),
                        heal=champion.get("heal"),
                        time_ccing_others=champion.get("time_ccing_others"),
                        op_score=champion.get("op_score"),
                        is_max_in_team_op_score=champion.get("is_max_in_team_op_score"),
                        physical_damage_taken=champion.get("physical_damage_taken"),
                        damage_dealt_to_champions=champion.get(
                            "damage_dealt_to_champions"
                        ),
                        physical_damage_dealt_to_champions=champion.get(
                            "physical_damage_dealt_to_champions"
                        ),
                        magic_damage_dealt_to_champions=champion.get(
                            "magic_damage_dealt_to_champions"
                        ),
                        damage_dealt_to_objectives=champion.get(
                            "damage_dealt_to_objectives"
                        ),
                        damage_dealt_to_turrets=champion.get("damage_dealt_to_turrets"),
                        damage_self_mitigated=champion.get("damage_self_mitigated"),
                        max_largest_multi_kill=champion.get("max_largest_multi_kill"),
                        max_largest_critical_strike=champion.get(
                            "max_largest_critical_strike"
                        ),
                        max_largest_killing_spree=champion.get(
                            "max_largest_killing_spree"
                        ),
                        snowball_throws=champion.get("snowball_throws"),
                        snowball_hits=champion.get("snowball_hits"),
                    )
                )

            # Ensure 'recent_game_stats' is handled properly
            recent_game_stats = (
                self.get_recent_games() if hasattr(self, "get_recent_games") else None
            )

        except Exception as e:
            self.logger.error(f"Error parsing some summoner data: {e}")
            pass

        return Summoner(
            id=summoner_data.get("id"),
            summoner_id=summoner_data.get("summoner_id"),
            acct_id=summoner_data.get("acct_id"),
            puuid=summoner_data.get("puuid"),
            game_name=summoner_data.get("game_name"),
            tagline=summoner_data.get("tagline"),
            name=summoner_data.get("name"),
            internal_name=summoner_data.get("internal_name"),
            profile_image_url=summoner_data.get("profile_image_url"),
            level=summoner_data.get("level"),
            updated_at=summoner_data.get("updated_at"),
            renewable_at=summoner_data.get("renewable_at"),
            previous_seasons=previous_seasons,
            league_stats=league_stats,
            most_champions=most_champions,
            recent_game_stats=recent_game_stats,
        )

    def search(
        self, summoner_names: str | list[str], region=Region.NA
    ) -> list[Summoner]:
        """
        Search for a single or multiple summoner(s) on OPGG.

        ### Args:
            summoner_names : `str` | `list[str]`
                Pass either a `str` (comma seperated) or `list[str]` of summoner names + regional identifier (#NA1, #EUW, etc).

            region : `Region, optional`
                Pass the region you want to search in. Defaults to "NA".

        ### Returns:
            `list[Summoner]` | `str` : A single or list of Summoner objects, or a string if no summoner(s) were found.

        """

        if isinstance(summoner_names, str) and "," in summoner_names:
            summoner_names = summoner_names.split(",")
        elif isinstance(summoner_names, str):
            summoner_names = [summoner_names]

        # General flow of cache retrieval:
        # 1. Pull from cache db
        #   -> If found, add to list of cached summoner ids, and below iterate over and set the summoner id property
        #   -> As an extension of the above, these requests would go directly to the api to pull summary/full data
        #   -> If not found, add to list of summoner names to query
        # 2. Build the summoner objects accordingly
        cached_summoner_ids = []
        uncached_summoners = []

        # TODO: Cache get/set logic needs to be reworked to account for regional identifiers. Currently, you could have 2 of the same users on different regions and the cache will just return whatever the first entry that was cached would've been. This is not ideal, perhaps I should force people to append the regional identifiers? This would result in unique entries in the cache...
        for summoner_name in summoner_names:
            if "#" not in summoner_name:
                raise Exception(
                    f'No regional identifier was found for query: "{summoner_name}". Please include the identifier as well and try again. (#NA1, #EUW, etc.)'
                )
            cached_id = self.cacher.get_summoner_id(summoner_name)
            if cached_id:
                cached_summoner_ids.append(cached_id)
            else:
                uncached_summoners.append(summoner_name)

        # pass only uncached summoners to get_page_props()
        page_props = Utils.get_page_props(uncached_summoners, region)

        self.logger.debug(
            f"\n********PAGE_PROPS_START********\n{page_props}\n********PAGE_PROPS_STOP********"
        )

        if len(uncached_summoners) > 0:
            self.logger.info(
                f"No cache for {len(uncached_summoners)} summoners: {uncached_summoners}, fetching... (using get_page_props() site scraper)"
            )
        if len(cached_summoner_ids) > 0:
            self.logger.info(
                f"Cache found for {len(cached_summoner_ids)} summoners: {cached_summoner_ids}, fetching... (using get_summoner() api)"
            )

        # Query cache for champs and seasons
        cached_seasons = self.cacher.get_all_seasons()
        cached_champions = self.cacher.get_all_champs()

        # If we found some cached seasons/champs, use them, otherwise fetch and cache them.
        if cached_seasons:
            self.all_seasons = cached_seasons
        else:
            self.all_seasons = Utils.get_all_seasons(self.region, page_props)
            self.cacher.insert_all_seasons(self.all_seasons)

        if cached_champions:
            self.all_champions = cached_champions
        else:
            self.all_champions = Utils.get_all_champions(self.region, page_props)
            self.cacher.insert_all_champs(self.all_champions)

        # todo: if more than 5 summoners are passed, break into 5s and iterate over each set
        # note: this would require calls to the refresh_api_url() method each iteration?

        # set the region to the passed one. this is what get_summoner() relies on
        self.region = region

        # bit of weirdness around generic usernames. If you pass "abc" for example, it will return multiple summoners in the page props.
        # To help, we will check against opgg's "internal_name" property, which seems to be the username.lower() with spaces removed.
        summoners = []
        for summoner_name in summoner_names:
            # if there are multiple search results for a SINGLE summoner_name, query MUST include the regional identifier
            if len(page_props["summoners"]) > 1 and "#" in summoner_name:
                logging.debug(
                    f"MULTI-RESULT | page_props->summoners: {page_props['summoners']}"
                )
                only_summoner_name, only_region = summoner_name.split("#")
                for summoner in page_props["summoners"]:
                    if (
                        only_summoner_name.strip() == summoner["game_name"]
                        and only_region.strip() == summoner["tagline"]
                    ):
                        self.summoner_id = summoner["summoner_id"]
                        break

            elif len(page_props["summoners"]) > 1 and "#" not in summoner_name:
                raise Exception(
                    f'Multiple search results were returned for "{summoner_name}". Please include the identifier as well and try again. (#NA1, #EUW, etc.)'
                )

            elif len(page_props["summoners"]) == 1:
                self.summoner_id = page_props["summoners"][0]["summoner_id"]

            summoner = self.get_summoner()
            summoners.append(summoner)
            self.logger.info(
                f"Summoner object built for: {summoner.name} ({summoner.summoner_id}), caching..."
            )
            self.cacher.insert_summoner(summoner.name, summoner.summoner_id)

        # cached summoners go straight to api
        for _cached_summoner_id in cached_summoner_ids:
            self.summoner_id = _cached_summoner_id
            summoner = self.get_summoner()
            summoners.append(summoner)
            self.logger.info(
                f"Summoner object built for: {summoner.name} ({summoner.summoner_id})"
            )

        # todo: add custom exceptions instead of this.
        # todo: raise SummonerNotFound exception
        if len(summoners) == 0:
            raise Exception(f"No summoner(s) matching {summoner_names} were found...")

        return summoners if len(summoners) > 1 else summoners[0]

    def get_recent_games(
        self,
        results: int = 10,
        game_type: Literal["total", "ranked", "normal"] = "total",
        return_content_only=False,
    ) -> list[Game]:
        """
        Get

        Args:
            results (int, optional): _description_. Defaults to 10.
            game_type (Literal[&quot;total&quot;, &quot;ranked&quot;, &quot;normal&quot;], optional): _description_. Defaults to "total".
            return_content_only (bool, optional): _description_. Defaults to False.

        Returns:
            list[Game]: _description_
        """

        recent_games = []
        res = requests.get(
            f"{self._games_api_url}?&limit={results}&game_type={game_type}",
            headers=self.headers,
        )

        self.logger.debug(res.text)

        if res.status_code == 200:
            self.logger.info(
                f"Request to OPGG GAME_API was successful, parsing data (Content Length: {len(res.text)})..."
            )
            game_data: Game = json.loads(res.text)["data"]
        else:
            res.raise_for_status()

        if return_content_only:
            return game_data

        try:
            game: dict
            for game in game_data:

                participants = []

                participant: dict
                participant_summoner: dict
                participant_stats: dict
                participant_tier_info: dict
                for participant in game.get("participants", []):
                    # Assign 'summoner' data to a variable
                    participant_summoner = participant.get("summoner", {})

                    # Create Summoner object with safe key access
                    summoner = Summoner(
                        id=participant_summoner.get("id"),
                        summoner_id=participant_summoner.get("summoner_id"),
                        acct_id=participant_summoner.get("acct_id"),
                        puuid=participant_summoner.get("puuid"),
                        game_name=participant_summoner.get("game_name"),
                        tagline=participant_summoner.get("tagline"),
                        name=participant_summoner.get("name"),
                        internal_name=participant_summoner.get("internal_name"),
                        profile_image_url=participant_summoner.get("profile_image_url"),
                        level=participant_summoner.get("level"),
                        updated_at=participant_summoner.get("updated_at"),
                        renewable_at=participant_summoner.get("renewable_at"),
                    )

                    # Assign 'stats' and 'tier_info' data to variables
                    participant_stats = participant.get("stats", {})
                    participant_tier_info = participant.get("tier_info", {})

                    # Create Participant object
                    participants.append(
                        Participant(
                            summoner=summoner,
                            participant_id=participant.get("participant_id"),
                            champion_id=participant.get("champion_id"),
                            team_key=participant.get("team_key"),
                            position=participant.get("position"),
                            role=participant.get("role"),
                            items=participant.get("items"),
                            trinket_item=participant.get("trinket_item"),
                            rune={
                                participant.get("rune", {}).get("primary_page_id"),
                                participant.get("rune", {}).get("primary_rune_id"),
                                participant.get("rune", {}).get("secondary_page_id"),
                            },  # Consider creating a Rune object in the future
                            spells=participant.get("spells"),
                            stats=Stats(
                                champion_level=participant_stats.get("champion_level"),
                                damage_self_mitigated=participant_stats.get(
                                    "damage_self_mitigated"
                                ),
                                damage_dealt_to_objectives=participant_stats.get(
                                    "damage_dealt_to_objectives"
                                ),
                                damage_dealt_to_turrets=participant_stats.get(
                                    "damage_dealt_to_turrets"
                                ),
                                magic_damage_dealt_player=participant_stats.get(
                                    "magic_damage_dealt_player"
                                ),
                                physical_damage_taken=participant_stats.get(
                                    "physical_damage_taken"
                                ),
                                physical_damage_dealt_to_champions=participant_stats.get(
                                    "physical_damage_dealt_to_champions"
                                ),
                                total_damage_taken=participant_stats.get(
                                    "total_damage_taken"
                                ),
                                total_damage_dealt=participant_stats.get(
                                    "total_damage_dealt"
                                ),
                                total_damage_dealt_to_champions=participant_stats.get(
                                    "total_damage_dealt_to_champions"
                                ),
                                largest_critical_strike=participant_stats.get(
                                    "largest_critical_strike"
                                ),
                                time_ccing_others=participant_stats.get(
                                    "time_ccing_others"
                                ),
                                vision_score=participant_stats.get("vision_score"),
                                vision_wards_bought_in_game=participant_stats.get(
                                    "vision_wards_bought_in_game"
                                ),
                                sight_wards_bought_in_game=participant_stats.get(
                                    "sight_wards_bought_in_game"
                                ),
                                ward_kill=participant_stats.get("ward_kill"),
                                ward_place=participant_stats.get("ward_place"),
                                turret_kill=participant_stats.get("turret_kill"),
                                barrack_kill=participant_stats.get("barrack_kill"),
                                kill=participant_stats.get("kill"),
                                death=participant_stats.get("death"),
                                assist=participant_stats.get("assist"),
                                largest_multi_kill=participant_stats.get(
                                    "largest_multi_kill"
                                ),
                                largest_killing_spree=participant_stats.get(
                                    "largest_killing_spree"
                                ),
                                minion_kill=participant_stats.get("minion_kill"),
                                neutral_minion_kill_team_jungle=participant_stats.get(
                                    "neutral_minion_kill_team_jungle"
                                ),
                                neutral_minion_kill_enemy_jungle=participant_stats.get(
                                    "neutral_minion_kill_enemy_jungle"
                                ),
                                neutral_minion_kill=participant_stats.get(
                                    "neutral_minion_kill"
                                ),
                                gold_earned=participant_stats.get("gold_earned"),
                                total_heal=participant_stats.get("total_heal"),
                                result=participant_stats.get("result"),
                                op_score=participant_stats.get("op_score"),
                                op_score_rank=participant_stats.get("op_score_rank"),
                                is_opscore_max_in_team=participant_stats.get(
                                    "is_opscore_max_in_team"
                                ),
                                lane_score=participant_stats.get("lane_score"),
                                op_score_timeline=participant_stats.get(
                                    "op_score_timeline"
                                ),
                                op_score_timeline_analysis=participant_stats.get(
                                    "op_score_timeline_analysis"
                                ),
                            ),
                            tier_info=Tier(
                                tier=participant_tier_info.get("tier"),
                                division=participant_tier_info.get("division"),
                                lp=participant_tier_info.get("lp"),
                                level=participant_tier_info.get("level"),
                                tier_image_url=participant_tier_info.get(
                                    "tier_image_url"
                                ),
                                border_image_url=participant_tier_info.get(
                                    "border_image_url"
                                ),
                            ),
                        )
                    )

                teams = []

                team: dict
                team_game_stat: dict
                game_queue_info: dict
                game_average_tier_info: dict
                for team in game.get("teams", []):
                    team_game_stat = team.get("game_stat", {})
                    teams.append(
                        Team(
                            key=team.get("key"),
                            game_stat=GameStats(
                                is_win=team_game_stat.get("is_win"),
                                champion_kill=team_game_stat.get("champion_kill"),
                                champion_first=team_game_stat.get("champion_first"),
                                inhibitor_kill=team_game_stat.get("inhibitor_kill"),
                                inhibitor_first=team_game_stat.get("inhibitor_first"),
                                rift_herald_kill=team_game_stat.get("rift_herald_kill"),
                                rift_herald_first=team_game_stat.get(
                                    "rift_herald_first"
                                ),
                                dragon_kill=team_game_stat.get("dragon_kill"),
                                dragon_first=team_game_stat.get("dragon_first"),
                                baron_kill=team_game_stat.get("baron_kill"),
                                baron_first=team_game_stat.get("baron_first"),
                                tower_kill=team_game_stat.get("tower_kill"),
                                tower_first=team_game_stat.get("tower_first"),
                                horde_kill=team_game_stat.get("horde_kill"),
                                horde_first=team_game_stat.get("horde_first"),
                                is_remake=team_game_stat.get("is_remake"),
                                death=team_game_stat.get("death"),
                                assist=team_game_stat.get("assist"),
                                gold_earned=team_game_stat.get("gold_earned"),
                                kill=team_game_stat.get("kill"),
                            ),
                            banned_champions=team.get("banned_champions", []),
                        )
                    )

                # Assign 'queue_info' and 'average_tier_info' data to variables
                game_queue_info = game.get("queue_info", {})
                game_average_tier_info = game.get("average_tier_info", {})

                # Create Game object
                tmp_game = Game(
                    id=game.get("id"),
                    created_at=game.get("created_at"),
                    game_map=game.get("game_map"),
                    queue_info=QueueInfo(
                        id=game_queue_info.get("id"),
                        queue_translate=game_queue_info.get("queue_translate"),
                        game_type=game_queue_info.get("game_type"),
                    ),
                    version=game.get("version"),
                    game_length_second=game.get("game_length_second"),
                    is_remake=game.get("is_remake"),
                    is_opscore_active=game.get("is_opscore_active"),
                    is_recorded=game.get("is_recorded"),
                    record_info=game.get("record_info"),
                    average_tier_info=Tier(
                        tier=game_average_tier_info.get("tier"),
                        division=game_average_tier_info.get("division"),
                        tier_image_url=game_average_tier_info.get("tier_image_url"),
                        border_image_url=game_average_tier_info.get("border_image_url"),
                    ),
                    participants=participants,
                    teams=teams,
                    memo=game.get("memo"),
                    myData=Participant(
                        summoner=Summoner(
                            id=game.get("myData", {}).get("summoner", {}).get("id"),
                            summoner_id=game.get("myData", {})
                            .get("summoner", {})
                            .get("summoner_id"),
                            acct_id=game.get("myData", {})
                            .get("summoner", {})
                            .get("acct_id"),
                            puuid=game.get("myData", {})
                            .get("summoner", {})
                            .get("puuid"),
                            game_name=game.get("myData", {})
                            .get("summoner", {})
                            .get("game_name"),
                            tagline=game.get("myData", {})
                            .get("summoner", {})
                            .get("tagline"),
                            name=game.get("myData", {}).get("summoner", {}).get("name"),
                            internal_name=game.get("myData", {})
                            .get("summoner", {})
                            .get("internal_name"),
                            profile_image_url=game.get("myData", {})
                            .get("summoner", {})
                            .get("profile_image_url"),
                            level=game.get("myData", {})
                            .get("summoner", {})
                            .get("level"),
                            updated_at=game.get("myData", {})
                            .get("summoner", {})
                            .get("updated_at"),
                            renewable_at=game.get("myData", {})
                            .get("summoner", {})
                            .get("renewable_at"),
                        ),
                        participant_id=game.get("myData", {}).get("participant_id"),
                        champion_id=game.get("myData", {}).get("champion_id"),
                        team_key=game.get("myData", {}).get("team_key"),
                        position=game.get("myData", {}).get("position"),
                        role=game.get("myData", {}).get("role"),
                        items=game.get("myData", {}).get("items"),
                        trinket_item=game.get("myData", {}).get("trinket_item"),
                        rune={
                            game.get("myData", {})
                            .get("rune", {})
                            .get("primary_page_id"),
                            game.get("myData", {})
                            .get("rune", {})
                            .get("primary_rune_id"),
                            game.get("myData", {})
                            .get("rune", {})
                            .get("secondary_page_id"),
                        },  # Consider creating a Rune object in the future
                        spells=game.get("myData", {}).get("spells"),
                        stats=Stats(
                            champion_level=game.get("myData", {})
                            .get("stats", {})
                            .get("champion_level"),
                            damage_self_mitigated=game.get("myData", {})
                            .get("stats", {})
                            .get("damage_self_mitigated"),
                            damage_dealt_to_objectives=game.get("myData", {})
                            .get("stats", {})
                            .get("damage_dealt_to_objectives"),
                            damage_dealt_to_turrets=game.get("myData", {})
                            .get("stats", {})
                            .get("damage_dealt_to_turrets"),
                            magic_damage_dealt_player=game.get("myData", {})
                            .get("stats", {})
                            .get("magic_damage_dealt_player"),
                            physical_damage_taken=game.get("myData", {})
                            .get("stats", {})
                            .get("physical_damage_taken"),
                            physical_damage_dealt_to_champions=game.get("myData", {})
                            .get("stats", {})
                            .get("physical_damage_dealt_to_champions"),
                            total_damage_taken=game.get("myData", {})
                            .get("stats", {})
                            .get("total_damage_taken"),
                            total_damage_dealt=game.get("myData", {})
                            .get("stats", {})
                            .get("total_damage_dealt"),
                            total_damage_dealt_to_champions=game.get("myData", {})
                            .get("stats", {})
                            .get("total_damage_dealt_to_champions"),
                            largest_critical_strike=game.get("myData", {})
                            .get("stats", {})
                            .get("largest_critical_strike"),
                            time_ccing_others=game.get("myData", {})
                            .get("stats", {})
                            .get("time_ccing_others"),
                            vision_score=game.get("myData", {})
                            .get("stats", {})
                            .get("vision_score"),
                            vision_wards_bought_in_game=game.get("myData", {})
                            .get("stats", {})
                            .get("vision_wards_bought_in_game"),
                            sight_wards_bought_in_game=game.get("myData", {})
                            .get("stats", {})
                            .get("sight_wards_bought_in_game"),
                            ward_kill=game.get("myData", {})
                            .get("stats", {})
                            .get("ward_kill"),
                            ward_place=game.get("myData", {})
                            .get("stats", {})
                            .get("ward_place"),
                            turret_kill=game.get("myData", {})
                            .get("stats", {})
                            .get("turret_kill"),
                            barrack_kill=game.get("myData", {})
                            .get("stats", {})
                            .get("barrack_kill"),
                            kill=game.get("myData", {}).get("stats", {}).get("kill"),
                            death=game.get("myData", {}).get("stats", {}).get("death"),
                            assist=game.get("myData", {})
                            .get("stats", {})
                            .get("assist"),
                            largest_multi_kill=game.get("myData", {})
                            .get("stats", {})
                            .get("largest_multi_kill"),
                            largest_killing_spree=game.get("myData", {})
                            .get("stats", {})
                            .get("largest_killing_spree"),
                            minion_kill=game.get("myData", {})
                            .get("stats", {})
                            .get("minion_kill"),
                            neutral_minion_kill_team_jungle=game.get("myData", {})
                            .get("stats", {})
                            .get("neutral_minion_kill_team_jungle"),
                            neutral_minion_kill_enemy_jungle=game.get("myData", {})
                            .get("stats", {})
                            .get("neutral_minion_kill_enemy_jungle"),
                            neutral_minion_kill=game.get("myData", {})
                            .get("stats", {})
                            .get("neutral_minion_kill"),
                            gold_earned=game.get("myData", {})
                            .get("stats", {})
                            .get("gold_earned"),
                            total_heal=game.get("myData", {})
                            .get("stats", {})
                            .get("total_heal"),
                            result=game.get("myData", {})
                            .get("stats", {})
                            .get("result"),
                            op_score=game.get("myData", {})
                            .get("stats", {})
                            .get("op_score"),
                            op_score_rank=game.get("myData", {})
                            .get("stats", {})
                            .get("op_score_rank"),
                            is_opscore_max_in_team=game.get("myData", {})
                            .get("stats", {})
                            .get("is_opscore_max_in_team"),
                            lane_score=game.get("myData", {})
                            .get("stats", {})
                            .get("lane_score"),
                            op_score_timeline=game.get("myData", {})
                            .get("stats", {})
                            .get("op_score_timeline"),
                            op_score_timeline_analysis=game.get("myData", {})
                            .get("stats", {})
                            .get("op_score_timeline_analysis"),
                        ),
                        tier_info=Tier(
                            tier=game.get("myData", {})
                            .get("tier_info", {})
                            .get("tier"),
                            division=game.get("myData", {})
                            .get("tier_info", {})
                            .get("division"),
                            lp=game.get("myData", {}).get("tier_info", {}).get("lp"),
                            level=game.get("myData", {})
                            .get("tier_info", {})
                            .get("level"),
                            tier_image_url=game.get("myData", {})
                            .get("tier_info", {})
                            .get("tier_image_url"),
                            border_image_url=game.get("myData", {})
                            .get("tier_info", {})
                            .get("border_image_url"),
                        ),
                    ),
                )

                recent_games.append(tmp_game)

            return recent_games

        except:
            self.logger.error(
                f"Unable to create game object, see trace: \n{traceback.format_exc()}"
            )
            pass



# from opgg.summoner import Summoner, Game
# from opgg.season import Season, SeasonInfo
# from opgg.champion import ChampionStats, Champion, Spell, Passive, Skin, Price
# from opgg.league_stats import LeagueStats, Tier, QueueInfo
# from opgg.params import Region, By
# from opgg.cacher import Cacher


from pydantic import BaseModel

from opgg.v2.params import Region
from opgg.v2.summoner import Summoner


class SearchResult(BaseModel):
    """
    #### Represents a search result from the OPGG API.

    ##### Properties:
        `region`: The region of the summoner.
        `summoner`: The summoner result.
    """

    region: Region
    summoner: Summoner

    class Config:
        arbitrary_types_allowed = True

    def __str__(self):
        return f"[{str(self.region):4}] {f'{self.summoner.game_name} #{self.summoner.tagline}':<25} | Level: {str(self.summoner.level):<4} [Summoner ID: {self.summoner.summoner_id}]"


from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()

long_description = (here / "README.md").read_text(encoding="utf-8")

setup(
    name="opgg.py",
    version="2.0.3",
    description="An unofficial Python library for scraping/accessing data from OP.GG",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ShoobyDoo/OPGG.py",
    author="ShoobyDoo",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Programming Language :: Python :: 3 :: Only",
    ],
    keywords="opgg, league-of-legends, web-scraping, summoner, data, riot-api",
    packages=find_packages(),
    python_requires=">=3.11, <4",
    install_requires=["aiohttp", "pydantic", "fake-useragent"],
    project_urls={
        "Bug Reports": "https://github.com/ShoobyDoo/OPGG.py/issues",
        "Source": "https://github.com/ShoobyDoo/OPGG.py",
    },
)